;============================================================
; SoundBlaster Autodetect Routine
;
; Author   : Bjorge Dijkstra (bjorge@gmx.net)
; Date     : 03-03-2000
; version  : 1.0
;============================================================

.MODEL TINY
.386


code32  segment para public use32
code32  ends

code32  segment para public use32
        assume cs:code32, ds:code32
        ORG 100H 

APP:

	jmp  start		; skip data and subroutines

include sbdsp.asm		; SoundBlaster DSP routines
include dma.asm			; DMA transfer routines

; some constants
	PIC1	equ  020h	; First  PIC (handles IRQ 0-7)
	PIC2	equ  0A0h	; Second PIC (handles IRQ 8-15)
	EOI	equ  020h	; End-Of-Interrupt cmd


; data
        Welcome         DB 'SBDETECT 0.2.0 by Bjorge Dijkstra (bjorge@gmx.net)',13,10,13,10
        		DB 'This is supposed to fully autodetect the settings of your SB soundcard',13,10
			DB 'that is, if you have one ofcourse ;)',13,10,13,10,0
	
	CheckFail	DB 'Failed',13,10,0
	CheckOk		DB 'OK',13,10,0
	
        Return          DB 13,10,0

	SBNotFound	DB 'SB not found.. :(',13,10,0
 	SBFound		DB 'I found a ',0
 	SBType		DB ' on :'
 	SbOnPort	DB 13,10,'PORT        : ',0
	SBOnIRQ		DB 13,10,'IRQ         :   ',0
	SBOnLoDMA	DB 13,10,'LoDMA       :   ',0
	SBOnHiDMA	DB 13,10,'HiDMA       :   ',0
	SBDSPVer	DB 13,10,'DSP Version : ',0
	
	SBModel		DD 0			; offset to modelstring
	DSPVMajor	DB 0
	DSPVMinor	DB 0	
	SBPort		DW 240h
	SBIRQ		DB 5
	SBLoDMA		DB 3
	SBHiDMA		DB 5
	
	SBModel1x	DB 'SoundBlaster 1.x',0
	SBModel20	DB 'SoundBlaster 2.0',0
	SBModelPro	DB 'SoundBlaster Pro',0
	SBModelPro2	DB 'SoundBlaster Pro II',0
	SBModelSB16	DB 'SoundBlaster 16 (or higher)',0
	SBUnkModel	DB 'Unknown Model (please report your SB type and DSP version to bjorge@gmx.net)',0
	
	saveIntSel	DW 0
        saveIntOff	DD 0
	IRQDetected	DB 0
	
        IRQs		DB 07h,05h,0Ah,02h		; irq's to check in preferred order
        LoDMAs		DB 01h,03h,00h			; dma's to check in preferred order        

        SB16IRQs	DB 02h,05h,07h,0Ah		; possible irq's on sb
        SB16LoDMAs	DB 00h,01h,02h,03h		; possible dma's
        SB16HiDMAs	DB 04h,05h,06h,07h		;
        
	IRQtoInt	DB 08h,09h,0Ah,0Bh,0Ch,0Dh,0Eh,0Fh,70h,71h,72h,73h,74h,75h,76h,77h	
	
;=============================================================
; SBISR
; -----
; SoundBlaster Interrupt Service Routine
; this interrupt is called after a completed dma transfer
; 
;=============================================================
sbisr:	
	cli
	push	eax
	push	edx
	push	edi

	mov	dx,SBPort
	add	dx,0eh
	in	al,dx				; ACk interrupt to SB by reading port 2xEh
	
	mov	IRQDetected,1
	
	mov	al,SBIRQ
	and	al,0F8h
	jz	sbisr_AckPIC1			; IRQ > 7 ?
	mov	dx,PIC2				; Yes, also ACK interrupt to PIC2
	mov	al,EOI
	out	dx,al
sbisr_AckPIC1:	
	mov	dx,PIC1				; ACK interrupt to PIC1
	mov	al,EOI
	out	dx,al

	pop	edi
	pop	edx
	pop	eax              
	sti
	iret
	

;=============================================================
; Setup and enable SB interrupt handler
;=============================================================
HookSBint:
	push	eax
	push	ebx
	push	ecx
	push	edx
	push	edi

        xor	ax,ax
	mov	al,SBIRQ
	mov	di,ax        
	mov	ah,[IRQtoINT + di]		; map IRQ to an interrupt
        mov	al,2ah				; save old int
	int	20h
	mov	saveIntSel,bx
	mov	saveIntOff,edi
	
        xor	ax,ax
	mov	al,SBIRQ
	mov	di,ax        
	mov	ah,[IRQtoINT + di]		; map IRQ to an interrupt
	mov	al,1				; set new int
	mov	bx,cs
	mov	edi,OFFSET sbisr
        int	20h

	mov	al,SBIRQ				; IRQ > 7?
	and	al,0F8h
	jnz	HookSBInt_OnPIC2		; yes, select PIC2
	mov	dx,PIC1+1			; else select PIC1
	mov	cl,SBIRQ
	jmp	HookSBInt_EnableInt
HookSBInt_OnPIC2:
	mov	dx,PIC2+1
	mov	cl,SBIRQ
	sub	cl,8
HookSBInt_EnableInt:
	mov	bl,1				; enable interrupt
	shl	bl,cl
	not	bl
	in	al,dx
	and	al,bl
	out	dx,al

	pop	edi
	pop	edx
	pop	ecx
	pop	ebx
	pop	eax
	ret

;=============================================================
; Restore old interrupt handler
;=============================================================
RestoreSBint:
	push	eax
	push	ebx
	push	ecx
	push	edx
	push	edi

	xor	ax,ax
	mov	al,SBIRQ
	mov	di,ax
	mov	ah,[IRQtoINT + di]		; map IRQ to an interrupt
	mov	al,1
	mov	bx,saveIntSel			; restore saved int
	mov	edi,saveIntOff
	int	20h

	mov	al,SBIRQ			; IRQ > 7?
	and	al,0F8h
	jnz	RestoreSBInt_OnPIC2		; yes, select PIC2
	mov	dx,PIC1+1			; else select PIC1
	mov	cl,SBIRQ
	jmp	RestoreSBInt_DisableInt
RestoreSBInt_OnPIC2:
	mov	dx,PIC2+1
	mov	cl,SBIRQ
	sub	cl,8
RestoreSBInt_DisableInt:
	mov	bl,1				; disable interrupt
	shl	bl,cl
	in	al,dx
	or	al,bl
	out	dx,al

	pop	edi
	pop	edx
	pop	ecx
	pop	ebx
	pop	eax
	ret


;=============================================================
; ShowSettings
;=============================================================
ShowSettings:
	push	eax
	push	ebx
	push	edi

	mov	al,4
	mov	edi,OFFSET SBFound
	int	20h
	
	mov	al,4
	mov	edi,SBModel
	int	20h
	
	mov	al,4
	mov	edi, OFFSET SBType
	int	20h
	
	mov	ax,0205h
	mov	bx,SBPort
	int	20h

	mov	al,4
	mov	edi,OFFSET SBOnIRQ
	int	20h
	mov	ax,0105h
	mov	bl,SBIRQ
	int	20h
	
	mov	al,4
	mov	edi,OFFSET SBOnLoDMA
	int	20h
	mov	ax,0105h
	mov	bl,SBLoDMA
	int	20h
	
	mov	al,4
	mov	edi,OFFSET SBOnHiDMA
	int	20h
	mov	ax,0105h
	mov	bl,SBHiDMA
	int	20h
	
	mov	al,4
	mov	edi,OFFSET SBDSPVer
	int	20h
	
	mov	bh,DSPVMajor
	mov	bl,DSPVMinor
	mov	ax,0205h
	int	20h
	
	

	mov	al,4
	mov	edi,OFFSET Return
	int	20h
	
	mov	al,4
	mov	edi,OFFSET Return
	int	20h

		
	
	pop	edi
	pop	ebx
	pop	eax
	ret
	
	
;=============================================================
; TestDMAIRQCombo
;=============================================================
	Sound		DB 500 dup(80h)	 ; test buffer 
 TestDMAIRQCombo:
 	push	eax
 	push	ebx
 	push	ecx
 	push	edx
 	push	edi
 	push	esi
 	
 	stc				 ; assume failure	
 	mov	IRQDetected,0
	call	HookSBint		 ; Hook SB interrupt

        mov	dx,SBPort		 ; 
	mov	ebx,8000		 ; Set sample rate to 8KHz
	call	DSPSetSampleRate
	
	mov	bx,cs		 	 ; Get base address of selector cs
	mov	al,2bh
	int	20h
	add	edi,OFFSET Sound	 ; Add offset to test data
	

        mov	al,dmaModeSINGLE	 ; setup DMA transfer of sound data
        or	al,dmaRdTransfer 	 ; SB reads from memory
	mov	ah,SBLoDMA		 ; Use this DMA channel
	mov	cx,500	 	 	 ; size of block
	call	DMAInitTransfer	 	 ; 

	mov	cx,500
	mov	dx,SBPort
	mov	al,014h			; setup SB for 8-bit Mono DMA transfer	
	call	DSPWrite		; write command
	mov	al,cl			; block size lower byte
	call	DSPWrite		; 
	mov	al,ch			; block size higher byte
	call	DSPWrite		; playing starts immediately after writing this byte

	mov	al,1ah			; delay
	mov	edi,4			; wait 100 msec
	int	20h			; transfer should take about 65 msec

	call	RestoreSBint

	pop	esi		
	pop	edi
	pop	edx
	pop	ecx
	pop	ebx
	pop	eax
	ret 
 

;=============================================================
; FindIRQandLoDMA (used if it's not a SB16+)
;============================================================= 
FindIRQandLoDMA:
	push	eax
	push	ebx
	push	ecx
	push	edx
	push	edi
	push	esi
	
	mov	di,0			; DMA index
	mov	si,0			; IRQ index
	
FILD_Testloop:

	mov	al,[LoDMAs+di]		; select a DMA channel
	mov	SBLoDMA,al
	
	
	mov	al,[IRQs+si]		; select an IRQ
	mov	SBIRQ,al
	
	call 	TestDMAIRQCombo		; test 'em
	
	cmp	IRQDetected,1		; found ?
	je	FILD_Testsuccess		; yes, we're done!
	
	push	edi
	mov	al,04h
	mov	edi,OFFSET CheckFail
	int	20h
	pop 	edi
	
	inc	si			; no, next irq
	cmp	si,6			; tried all irq's?
	jl	FILD_Testloop
	
	mov	si,0			; then next dma
	inc	di
	cmp	di,3			; tried all dma's?
	jge	FILD_Testfailed		; then we didn't find anything
	jmp	FILD_Testloop
	
FILD_Testfailed:
	mov	al,04h
	mov	edi,OFFSET CheckFail
	int	20h
	stc
        jmp	FILD_End

FILD_Testsuccess:
	mov	al,04h
	mov	edi,OFFSET CheckOk
	int	20h
	clc
	
FILD_End:
	pop	esi
	pop	edi
	pop	edx
	pop	ecx
	pop	ebx
	pop	eax	
 	ret
 	
 
;=============================================================
; SB16FindIRQandDMA
;=============================================================  
SB16FindIRQandDMA:
	push	eax			; save used registers
	push	ebx
	push	ecx
	push	edx
	push	edi
	
	mov	dx,SBPort		
	add	dx,04h			; select mixer index port
	mov	al,80h			; irq select register
	out	dx,al			; select

	xor	ax,ax
	inc	dx			; select mixer data port
	in	al,dx			; read irq select register
	
	mov	di,0		
	mov	cx,4			; get irq setting from al
IRQloop:
	test	al,1			; al : 01h = irq 2
	jnz	IRQOk			;      02h = irq 5
	shr	al,1			;      04h = irq 7
	inc	di			;      08h = irq 10
	loop	IRQloop
IRQOk:	
	mov	al,[SB16IRQs + di]
	mov	SBIRQ,al
	
	
	mov	dx,SBPort
	add	dx,04h			; select mixer index port
	mov	al,81h			; dma select register
	out	dx,al			; select
	
	xor	ax,ax
	inc	dx			; select mixer data port
	in	al,dx			; read dma select register
	and	al,11101011b		; mask out reserved dma's 2,4
	
	mov	bl,al			; al = LoDMA setting, bl = HiDMA setting
	shr	bl,4
	mov	cx,4
	mov	di,0			; get LoDMA setting from al
LoDMAloop:
	test	al,1			; al : 01h = dma 0
	jnz	LoDMAOk			;      02h = dma 1
	shr	al,1			;      04h = dma 2 (reserved)
	inc	di			;      08h = dma 3
	loop	LoDMAloop
LoDMAOk:		
	mov	al,[SB16LoDMAs + di]
	mov	SBLoDMA,al

	mov	cx,4
	mov	di,0			; get HiDMA from bl
HiDMAloop:
	test	bl,1			; bl : 01h = dma 4 (reserved)
	jnz	HiDMAOk			;      02h = dma 5
	shr	bl,1			;      04h = dma 6
	inc	di			;      05h = dma 7
	loop	HiDMAloop
HiDMAOk:	
	mov	al,[SB16HiDMAs + di]
	mov	SBHiDMA,al	
	
	pop	edi
	pop	edx
	pop	ecx
	pop	ebx
	pop	eax
	ret
	
;=============================================================
; GetSBModel
;=============================================================  	
GetSBModel:
	push	eax
	
	mov	ah,DSPVMajor
	mov	al,DSPVMinor
	
	cmp	ah,1		; 1.x ?
	jne	GSM_M2
	mov	SBModel,OFFSET SBModel1x
	jmp	GSM_End

GSM_M2:
	cmp	ah,2
	jne	GSM_M3
	mov	SBModel,OFFSET SBModel20
	jmp	GSM_End
	
GSM_M3:
	cmp	ah,3
	jne	GSM_M4
	cmp	al,0
	jne	GSM_M3b
	mov	SBModel,OFFSET SBModelPro	
	jmp	GSM_End
GSM_M3b:
	mov	SBModel,OFFSET SBModelPro2
	jmp	GSM_End
	
GSM_M4:
	cmp	ah,4
	jne	GSM_UnkModel
	mov	SBModel,OFFSET SBModelSB16
	jmp	GSM_End
GSM_UnkModel:
	mov	SBModel,OFFSET SBUnkModel
	
GSM_End:
	pop	eax
	ret
;=============================================================
;=============================================================
;
;  MAIN
;
;=============================================================
;=============================================================

start:
	mov	al,04h               	; Select System32 Service 4 (PrintText)
	mov	edi,OFFSET Welcome  	; Point EDI to our textline
	int	20h                 	; Do System32 Service

	call	DSPDetectBase		; find base port
	jc	SBDetect_Failed
	mov	SBPort,dx
	
	call	DSPGetVersion
	mov	DSPVMajor,ah
	mov	DSPVMinor,al
	
	call	GetSBModel
	
	cmp	ah,4			; Is it a SB16+ ?
	jl	NOSB16			; no then try alternative
	
	call	SB16FindIRQandDMA	; read SB16 settings from mixer chip
	
	jmp	ShowWhatWeGot
	
NOSB16:	
	
	call	DSPSpeakerOff		; Turn speakers off, because we're going to output
					; some test data, which can be noisy
	
        call    FindIRQandLoDMA		; Find IRQ and LoDMA
        jc	SBDetect_Failed
        
ShowWhatWeGot:
		
	call	ShowSettings	
	
	jmp	SBDetect_End
	        
SBDetect_Failed:
	
	mov  	al,04h
	mov  	edi,OFFSET SBNotFound
	int  	20h

SBDetect_End:

	retf
code32  ends

END APP

