/*
 *  Some Text Draw Functions
 */

#include "v2os.h"

//
// Gets the character at coordinates x, y
//
char getxchr( int x, int y)
{
	return peekb(0x18, 0xb8000+2*(x+y*80));
}

//
// Gets the color of the character at coordinates x, y
//
char getxcol( int x, int y)
{
	return peekb(0x18, 0xb8000+1+2*(x+y*80));
}


//
// Puts a character at coordinates x, y, with given color
// (from SimpleEdit)
//
void inline putcc(int x, int y, short c, short color)
{
	int offset=0xb8000+((x+y*80)<<1);

	asm("
		mov %2, %%ax
		mov %1, %%bx
		mov %%al, %%bh
		mov %0, %%eax
		mov %%bx, %%gs:(%%eax)
		"
		:
		: "rm" (offset), "rm" (c), "rm" (color)
		: "eax", "bx"
	);
}



//
// Puts a string at coordinates x, y, with given color
//
void putsc(int x, int y, unsigned char *msg, unsigned char color)
{
  int l, i;

  l = y*80+x;
  for (i=0; i<255 && msg[i]!='\0'; i++)
  {
      pokeb(0x18, 0xb8000+1+2*l, color);
      pokeb(0x18, 0xb8000+2*l, msg[i]);
      l++;
  }
}



//
// Fill the screen with given color
//
void clear(unsigned char color)
{

	int i;
	for (i=0;i<80*44;i++)
	{
	    pokeb(0x18,0xb8000+1+2*i,color);
	    pokeb(0x18,0xb8000+2*i,0);
	}
}



//
// Draw a window at x,y
//
void window(int x, int y, int width, int height, unsigned char color)
{
  int i, j;

  putcc(x, y, 0xda, color);              //top left rounded border
  putcc(x+width, y, 0xbf, color);        //top right rounded border
  putcc(x, y+height, 0xc0, color);       //down left rounded border
  putcc(x+width, y+height, 0xd9, color); //down right rounded border

  for (i=1; i<width; i++)  // horizontal bars
  {
      putcc(i+x, y, 0xc4, color);
      putcc(i+x, y+height, 0xc4, color);
      for (j=1; j<height; j++)
          putcc(i+x, j+y, 0, color); //fill window
  }
  for (i=1; i<height; i++)  //vertical bars
  {
      putcc(x, i+y, 0xb3, color);
      putcc(x+width, i+y, 0xb3, color);
  }
}



//
// Draw a button in x, y
// focus: 1 - have focus;  0 - no focus;
//
void button(int x, int y, unsigned char *caption, char focus)
{
  int lenght;

  lenght = v2_strlen(caption);
  window(x, y, lenght+3, 2, 0xcf);

  // text - blue on white if focus is true
  if (focus)
  {
     putcc(x+1, y+1, ' ',0xfc);
     putsc(x+2, y+1, caption, 0xfc);
     putcc(x+lenght+2, y+1, ' ',0xfc);
  }
  else
  {
     putcc(x+1, y+1, ' ',0xcf);
     putsc(x+2, y+1, caption, 0xcf);
     putcc(x+lenght+2, y+1, ' ',0xcf);
  }
}



//
//  Display a box with a message and buttons yes-no
//  and return which button was pressed
//
int msgboxyesno(int x, int y, unsigned char *txt)
{
  char key;
  int btnum, lenght;

  lenght = v2_strlen(txt);
  window(x, y, lenght+3, 6, 0xcf);

  putsc(x+2, y+1, txt, 0xcf); // message
  button(x+7, y+3, "Yes", 1); // button yes
  button(x+16, y+3, "No", 0); // button no
  btnum = 0;

  do
  {
     key = getkey();
     switch (key)
     {
        case '4': //left
                 button(x+7, y+3, "Yes", 1);
                 button(x+16, y+3, "No", 0);
                 btnum = 0;
		 break;
        case '6': //right
                 button(x+7, y+3, "Yes", 0);
                 button(x+16, y+3, "No", 1);
                 btnum = 1;
		 break;
     }
  }
  while (key != 0x0d);  // ENTER

  return btnum;
}


//
//Return to the terminal screen
//
void restoreterm()
{

  asm("
	mov $24, %%al
        int $0x20"
	:
	:
	: "eax"
     );

}
