/*
 edit.c - Main source file of SimpleEdit
 Date: 04/2000
 Author: mac (mac_mac@gmx.net)
 License: GPL
 Version: 0.6
 Update: www.geocities.com/boerdidrosi
*/

/*
  NOTE: since SimpleEdit is still BETA, and the sources might be a bit confusing. I'll order them soon. Maybe.
*/

/*
 v-pos:
	main()
	head
	InitDisplay()
*/

/*
TO DO:
   * Key-interrupt-handler => Enter ASCII-Characters by pressing the ALT-Key and some numbers
   * Tools => ASM/C Syntax Highlighting
   * Add insert-mode
   * Support delete-key
   * Add a replace function
   * EnterString/LoadFile: don't destroy old file if operation is cancled
   * Implement TAB-characters

As soon as a better filesystem and a improved MemoryManagement is in the Kernel:
   * Rewrite Memory-Management 
     => Split Text into 64K-Blocks that can be spread around in MEM
	( not necessary if paging will be avaiable )
     => Dynamic BackBuffer, with an auto-adapting size
   * Rewrite DiskIO
     => Rewrite save-function, allow overwriting of files
*/

#include "v2os.h"
#include "edit.h"

/********** VARIABLES SPECIFIC TO CURRENT TEXT ********************/
int TB_size;	//TextBuffer
int TB_ptr;

int LB_ptr;	//LineBuffer
int LB_size;

int line_count;

int text_size;
int text_style; //0=DOS, 1=UNIX

int Mark_Start;
int Mark_End;

int t_x,t_y;	// Position of Text inside the Window
int c_x,c_y;	// Position of Cursor inside the Text


/******* FILE INFORMATION **************/
char file_name[40] = "";
int file_disk;
int file_data_start;
int file_sector_count;
int file_attributes;
int file_rawlength;


/****** LINE AND TEXT MEMORY BUFFERS **************/

#define MaxWindows 10
int WINS_ptr;

#define BB_MaxSize 1024
int BB_size;	//BufferBuffer :)
int BB_ptr;
int BB_UsedSize;

/******* LINE AND TEXT INFORMATIONS ****************/
#define MaxLines (8192)

#define SecurityDistance (32)


/******* DISKBUFFER INFORMATION ************/
int DB_size=0;
int DB_offs=0;


/******* WINDOW DISPLAY INFORMATION ****************/
int w_x,w_y,w_w,w_h;	// Screenposition of text-window

int current_window;

char* Status_text="-";
int Status_color=0;

#define status_color_error (0+16*6)
#define status_color_normal (1+16*4)

/********** MOUSE INFORMATION **************************/
char* Mouse_ID = "V2_MOUSE";
int Mouse_Module; //0 if no mousemodule is installed, <>0 otherwise
int Mouse_X;
int Mouse_Y;
short Mouse_Buttons;

#define ME_Lost 0
#define ME_LButtonDown 1
#define ME_LButtonUp 2
#define ME_RButtonDown 3
#define ME_RButtonUp 4
#define ME_Move 5

/********** TEXT AND LINE BUFFER ACCSSES ******************/
#define GetLineStart(a) peekl(0x18,((a)<<3)+LB_ptr)
#define GetLineLength(a) peekl(0x18,((a)<<3)+LB_ptr+4)
#define SetLineStart(a,b) pokel(0x18,((a)<<3)+LB_ptr,b)
#define SetLineLength(a,b) pokel(0x18,((a)<<3)+LB_ptr+4,b)

#define GetChar(a) peekb(0x18,TB_ptr+(a))
#define SetChar(a,b) pokeb(0x18,TB_ptr+(a),b);

void inline PutChar(int x, int y, short c, short col)
{
	int offset=0xB8000+((x+y*80)<<1);

	asm("
		mov %2, %%ax
		mov %1, %%bx
		mov %%al, %%bh
		mov %0, %%eax
		mov %%bx, %%gs:(%%eax)
		"
		:
		: "rm" (offset), "rm" (c), "rm" (col)
		: "eax", "bx"
	);
}

void Window( int x, int y, int w, int h, int col, unsigned char* title, int flags ) //flags:1=scrollbar,2=bottom,4=cursorpos
{
	int i,j;

	if( (flags&4)==0 )
		for( i=x+1; i<x+w; i++ )
		{
			PutChar( i, y, 196, col );
			if( (flags&2)!=0)
				PutChar( i, y+h, 196, col );
		}
	else
		for( i=x+1; i<x+w; i++ )
		{
			if( (i<x+w-17)|(i>x+w-2) )	// TODO: This is sloooow, spee it up a bit!
				PutChar( i, y, 196, col );
			if( (flags&2)!=0)
				PutChar( i, y+h, 196, col );
		}


	if( (flags&1)==0 ) j=179;
		else j=186;

	for( i=y+1; i<y+h; i++)
	{
		PutChar( x, i, 179, col);
		PutChar( x+w, i, j, col); //scroll-bar
	}
	PutChar( x, y, 218, col );
	PutChar( x+w, y, 191, col );
	PutChar( x, y+h, 192, col );
	PutChar( x+w, y+h, 217, col );
	i=v2_strlen(title);

		//Print Title (if any)
	if( i>0 )
	{
		PutChar( x+2, y, 217, col);
		for( j=0; j<i; j++ )
			PutChar(x+3+j, y, *title++, 14+(col&0xF0) );
		PutChar( x+3+j, y, 192, col);
	}

		// Print Cursor position

		// Scroll-Bar
	if( (flags&1)!=0 )
	{
		PutChar( x+w, y+1, 197, col );
		PutChar( x+w, y+h-1, 197, col );
	}
}

void SetScrollBar( int x, int y, int w, int h, int col, int pos, int max )
{
	int v;		// = ((h-4)*(pos))/(max-1);	// pos<max!!, pos[0] = 0!!
	if( (max-1)==0 ) v=0;
		else v=((h-4)*(pos))/(max-1);

/*	if( max==0 ) v=0;
	else v=((h-4)*(pos))/max;
*/
	if( v>h-4 ) v=h-4;
	PutChar( x+w, y+2+v, 219, col );
}

void strcpy( char* source, char* dest )
{
//	term_puts( SCR_DEBUG, "strcpy() runs...\r\n");

	while( *source!=0 )
	{
		*dest=*source;
		source++;
		dest++;
	}
	*dest=0;

//	term_puts( SCR_DEBUG, "strcpy() ends...\r\n");
}

#include "FileIO.c"
#include "menu.c"
#include "Textuse.c"
#include "memman.c"
#include "mouse.c"
#include "windows.c"

void InputLoop()
{
	unsigned char c;
	int state=1;
	int update=1;

/*	int ctrl_down;
	int alt_down;
	int shift_down;

	char c2;
*/

	int Event;
	ProcessMenus();

	for(;;)
	{
		if( update==1 )
		switch( state )
		{
		case 1: Display(1); break;
		case 0: Display(2); break;
		}

		Event=0;
		for(;;)
		{
			c=getkey_nowait();
			if( c!=0 ) { Event=1; break; }
			if( (UpdateMousePos()==1)&(Mouse_Buttons!=0) ) {Event=2; break; }
			//Display(0);
			if( UpdateMouseButtons()!=0 ) {Event=3; break;}
		}
		if( c==27 ) getkey();

		update=1;

		if( Event==1 )
		{
			switch( state )
			{
			case 0: switch(MakeMenuChar( c ))
				{
				case 0: return;
				case 1: state=1; break;
				case 3: Display_Mouse(1);
					DisplayMainMenu(mx, my);
					update=0; //no need to redraw all (avoids flickering)
					Display_Mouse(0);
					break;
				}
				break;
			case 1: switch( ProcessEditChar( c ))
				{
				case 0: state=0; 
					mx=0;
					my=0;
					break;
				case 2: update=0;
					break;
				}
				break;
			}
		} else //Not a keypress but a mouseklick
		{
			switch( state )
			{
			case 0: switch( MakeMenuKlick( Event ) )
				{
				case 1: update=0;
					break;
				case 11:
				case 12:
				case 5: state=1; break;
				case 10: return; //quit
				}
				break;
			case 1: if( (Mouse_Y>0) | (Event==2) ) // Drag&Drop mode in text window
				{
					ProcessEditMouse( Event );
					break;
				}
				if( Mouse_Y==0 )
				{
					if( MakeMenuKlick( Event )==4 )
						state=0;
				}
				break;
			}
		} // mouseklick
	}
}

/*		alt_down = KeyDown( 56 );
		ctrl_down = KeyDown( 29 );
		shift_down = KeyDown( 42 );
*/
/*
		if( KeyDown(42)==1 ) c2='D'; else c2='U';
		PutChar( 3, 51, c2, 7 ); //left shift
		if( KeyDown(54)==1 ) c2='D'; else c2='U';
		PutChar( 4, 51, c2, 7 ); // right shift
		if( KeyDown(56)==1 ) c2='D'; else c2='U';
		PutChar( 6, 51, c2, 7 ); // alt
		if( KeyDown(29)==1 ) c2='D'; else c2='U';
		PutChar( 8, 51, c2, 7 ); // CTRL
*/

int main(void)
{
	term_puts( SCR_DEBUG, "\r\n=== SimpleEdit starting here ===\r\n" );

	puts("V2_SimpleEdit     v0.6          4/2000      written by mac (mac_mac@gmx.net)\r\n published under GPL\r\n");
	puts(" see http://www.geocities.com/boerdidrosi for updates\r\n\r\n");

	GetMemory(); // Allocate 256 KB right away...

//	Text_New();

	 term_puts( SCR_DEBUG, "win_init()...\r\n");
	win_init();


	 term_puts( SCR_DEBUG, "Mouse_Init()...\r\n");
	Mouse_Init();

	 term_puts( SCR_DEBUG, "InitDisplay()...\r\n");
	InitDisplay(1);

	 term_puts( SCR_DEBUG, "Starting InputLoop()...\r\n");
	InputLoop();

	RestoreDisplay();

	puts("\r\n");

	 term_puts( SCR_DEBUG, "=== SimpleEdit ending here ===\r\n" );
	return 0;
}
