/* testmod.c */

/*
 * 10/12/1999 Joost Faassen, Carlo Prelz
 * Copyright (C) V2_Lab Rotterdam
 *
 * A test module with examples of how to use library functions
 *
 * $Log$
 */

#include "v2os.h"

extern void s_mod_load(void);
extern void s_mod_unload(void);
extern void s_mod_service(void);

static const unsigned char mod_name[]="Test ";
static const unsigned char mod_desc[]="Converts time to secs and back";
static const unsigned char mod_copyright[]="Copyright (C) 1999 V2 Lab";

const char modhdr[4] __attribute__((section(".hdr")))="V2MD";
const char modID[8] __attribute__((section(".hdr")))="V2-Tm001";
const char modver_lo __attribute__((section(".hdr")))='\0';
const char modver_hi __attribute__((section(".hdr")))='\1';
const char osver_lo __attribute__((section(".hdr")))='\0';
const char osver_hi __attribute__((section(".hdr")))='\0';
void (*modload_ptr)(void) __attribute__((section(".hdr")))=s_mod_load;
void (*modunload_ptr)(void) __attribute__((section(".hdr")))=s_mod_unload;
void (*modservice_ptr)(void) __attribute__((section(".hdr")))=s_mod_service;
const char *modname __attribute__((section(".hdr")))=mod_name;
const char *moddesc __attribute__((section(".hdr")))=mod_desc;
const char *modcpyr __attribute__((section(".hdr")))=mod_copyright;

const unsigned short mon_yday[2][13]=
{
/* Normal years.  */
  {0,31,59,90,120,151,181,212,243,273,304,334,365},
/* Leap years.  */
  {0,31,60,91,121,152,182,213,244,274,305,335,366}
};

struct tm tms;
static unsigned char bfr[16];

void mod_run(void)
{
  
  puts("This is a \"module\" and cannot be run.\r\nPlease,use the 'loadmod' command instead!\r\r\n");

  offtime(92345600L,&tms);
  to_string(tms.tm_mday,bfr,2,0);
  puts(bfr);
  puts("/");
  to_string(tms.tm_mon+1,bfr,2,0);
  puts(bfr);
  puts("/");
  to_string(tms.tm_year,bfr,4,0);
  puts(bfr);
  puts(" ");
  to_string(tms.tm_hour,bfr,2,0);
  puts(bfr);
  puts(":");
  to_string(tms.tm_min,bfr,2,0);
  puts(bfr);
  puts(":");
  to_string(tms.tm_sec,bfr,2,0);
  puts(bfr);
  puts("\r\n");
}


void mod_load(void)
{
  announce("DateTime");
  puts("The module has been loaded!\r\n");
}

void mod_unload(void)
{
  int i;

  i+=2;
}

void mod_service(void)
{
  int i;

  i+=2;
}

static int offtime(const unsigned long t,struct tm *tp)
{
  long days,rem,y;
  const unsigned short *ip;

  days=t/SECS_PER_DAY;
  rem=t%SECS_PER_DAY;
  while(rem<0)
  {
    rem+=SECS_PER_DAY;
    --days;
  }
  while(rem>=SECS_PER_DAY)
  {
    rem-=SECS_PER_DAY;
    ++days;
  }
  tp->tm_hour=rem/SECS_PER_HOUR;
  rem%=SECS_PER_HOUR;
  tp->tm_min=rem/60;
  tp->tm_sec=rem%60;
  /* January 1,1970 was a Thursday.  */
  tp->tm_wday=(4+days)%7;
  if(tp->tm_wday<0)
    tp->tm_wday+=7;
  y=1970;
#define DIV(a,b) ((a)/(b)-((a)%(b)<0))
#define LEAPS_THRU_END_OF(y) (DIV(y,4)-DIV(y,100)+DIV(y,400))

  while(days< 0 || days>=(isleap(y) ? 366 : 365))
  {
/* Guess a corrected year, assuming 365 days per year.  */
    long yg=y+days/365-(days%365<0);
    
/* Adjust DAYS and Y to match the guessed year.  */
    days-=((yg-y)*365
           +LEAPS_THRU_END_OF(yg-1)
           -LEAPS_THRU_END_OF(y-1));
    y=yg;
  }
  tp->tm_year=y;

  tp->tm_yday=days;
  ip=mon_yday[isleap(y)];
  for (y=11;days<(long int)ip[y];--y)
    ;
  days-=ip[y];
  tp->tm_mon=y;
  tp->tm_mday=days+1;
  return 1;
}


