/* lib.c */

/*
 * 10/12/1999 Joost Faassen, Carlo Prelz
 * Copyright (C) V2_Lab Rotterdam
 *
 * Library of standard functions for developing modules and applications
 * for V2_OS in C/Linux
 *
 * $Log: lib.c,v $
 * Revision 1.1  1999/12/10 17:07:57  karl
 * Initial revision
 *
 */

#include "v2os.h"

int start=0x100; /* complains if not defined */
		/* must be "start" not "_start" if using DJGPP */

/*
 * When arrays are passed to lib routines, they seem to have to be declared static and out
 * of subroutines
 */

static unsigned char in_bfr[256];
static unsigned char out_bfr[256];

/*
 * Test routine - calls System32 service #2 (NULL function)
 */

inline void nullf(void)
{
  asm volatile (
		"mov $2,%%eax;int $0x20"
		: 
		: 
		: "eax"
		);
}

/*
 * Write directly to memory
 */

/*
movw %w0,%%fs	// Operand 0 -> FS
.byte 0x64	//FS Override prefix
movb %w1,(%k2)		//Operand 1 -> fs:(operand 2)
*/

inline void pokeb(unsigned short Seg, unsigned long Off, unsigned char Data)
{
  __asm__ __volatile__ (
    "movw %w0,%%fs \n.byte 0x64 \nmovb %1,(%k2)"
    :
    : "rm" (Seg), "ri" (Data), "r" (Off));
}

inline char peekb( unsigned short Seg, unsigned long Off)
{
	char c;
	asm("
		movw %1, %%ax
		movw %%ax, %%fs
		movl %2, %%eax
		movb %%fs:(%%eax), %%al
		movb %%al, %0
		"
		: "=m" (c)
		: "rm" (Seg), "rm" (Off)
		: "eax"
	);

	return c;
}

int peekl( unsigned short Seg, unsigned long Off)
{
	int i;

	asm("
		movw %1, %%ax
		movw %%ax, %%fs
		movl %2, %%eax
		movl %%fs:(%%eax), %%eax
		movl %%eax, %0
		"
		: "=m" (i)
		: "m" (Seg), "m" (Off)
		: "eax"
	);

	return i;
}

void pokel( unsigned short Seg, unsigned long Off, long data )
{
	asm("
		mov %0, %%ax
		mov %%ax, %%fs
		mov %1, %%eax
		mov %2, %%ebx
		mov %%ebx, %%fs:(%%eax)
		"
		:
		: "m" (Seg), "m" (Off), "m" (data)
		: "eax", "ebx"
	);
}

/*
 * Write a string to the main terminal. User should add \r\n
 */

inline void puts(unsigned char *msg)
{
  int i;

  for(i=0;i<255 && msg[i]!='\0';i++)
    in_bfr[i]=msg[i];
  in_bfr[i]='\0';
  
  asm("mov $4,%%eax;mov %0,%%edi;int $0x20"
      : /* no outputs */
      : "m" (in_bfr)
			: "eax", "edi"
      );
}

/*
 * Write a string to a given screen. User should add \r\n
 * <screen> should be set to one of the predefined variables
 * in v2os.h (SCR_TERMINAL, etc)
 */

inline void term_puts(unsigned char screen,unsigned char *msg)
{
  int i;

  for(i=0;i<255 && msg[i]!='\0';i++)
    in_bfr[i]=msg[i];
  in_bfr[i]='\0';
  
  asm("mov $38,%%eax;mov %0,%%edx;mov %1,%%edi;int $0x20"
      : /* no outputs */
      : "m" (screen), "m" (in_bfr)
			: "eax", "edx", "edi"
      );
}

/*
 * Pauses for given amount of 25ths of second
 */

void delay(unsigned long sec_25ths)
{
  asm("mov $26,%%eax;mov %0,%%edi;int $0x20"
      : /* no outputs */
      : "m" (sec_25ths)
			: "eax", "edi"
      );
}

/*
 * Print a byte, a short or a long to main terminal in hex
 */

void hexprintb(unsigned char val)
{
  asm("mov $0x105,%%eax;mov %0,%%ebx;int $0x20"
      : /* no outputs */
      : "m" (val)
			: "eax", "ebx"
      );
}
void hexprints(unsigned short val)
{
  asm("mov $0x205,%%eax;mov %0,%%ebx;int $0x20"
      : /* no outputs */
      : "m" (val)
			: "eax", "ebx"
      );
}
void hexprintl(unsigned long val)
{
  asm("mov $0x405,%%eax;mov %0,%%ebx;int $0x20"
      : /* no outputs */
      : "m" (val)
			: "eax", "ebx"
      );
}

/*
 * Print a byte, a short or a long to a screen in hex. <screen>
 * is set as in term_puts above
 */

void term_hexprintb(unsigned char screen,unsigned char val)
{
  asm("mov $0x127,%%eax;mov %0,%%edx;mov %1,%%ebx;int $0x20"
      : /* no outputs */
      : "m" (screen), "m" (val)
			: "eax", "ebx"
      );
}
void term_hexprints(unsigned char screen,unsigned short val)
{
  asm("mov $0x227,%%eax;mov %0,%%edx;mov %1,%%ebx;int $0x20"
      : /* no outputs */
      : "m" (screen), "m" (val)
			: "eax", "ebx"
      );
}
void term_hexprintl(unsigned char screen,unsigned long val)
{
  asm("mov $0x427,%%eax;mov %0,%%edx;mov %1,%%ebx;int $0x20"
      : /* no outputs */
      : "m" (screen), "m" (val)
			: "eax", "ebx"
      );
}

/*
 * Waits for a keypress. Returns the character
 */

unsigned char getkey(void)
{
	unsigned long l;
	
  asm("mov $23,%%eax;int $0x20;
mov $29,%%eax;int $0x20;
mov %%eax,%0"
			: "=m" (l)
			:
			: "eax"
			);

	return (unsigned char)l;
}

unsigned char getkey_nowait()
{
	unsigned long l;
	
  asm("mov $29,%%eax;int $0x20;mov %%eax,%0"
			: "=m" (l)
			:
			: "eax"
			);

	return (unsigned char)l;
}

/*
 * Reads a string from terminal. Must pass buffer with enough space
 */

void gets(unsigned char *bfr)
{
	int i;
	
  asm("mov $40,%%eax;mov %0,%%edi;int $0x20"
      : /* no outputs */
      : "m" (out_bfr)
			: "eax", "edi"
      );

	for(i=0;i<256 && out_bfr[i];i++)
		bfr[i]=out_bfr[i];
	bfr[i]='\0';
}

/*
 * Executes a V2_OS shell command
 */

void shell_command(unsigned char *cmd)
{
	int i;

  for(i=0;i<255 && cmd[i]!='\0';i++)
    in_bfr[i]=cmd[i];
  in_bfr[i]='\0';
	
  asm("mov $25,%%eax;mov %0,%%edi;int $0x20"
      : /* no outputs */
      : "m" (in_bfr)
			: "eax", "edi"
      );
}

/*
 * C routines...
 *
 * Convert from number to string.
 * val: input value
 * bfr: resulting string. Buffer is passed from caller.
 * len: length of the resulting string. If number is smaller,
 * string is completed with 0's,
 */

void num_to_string(int val,unsigned char *bfr,int len,unsigned char flg_cr)
{
  int i;
  
  for(i=len-1;i>=0;i--)
  {
    bfr[i]='0'+(val%10);
    val/=10;
  }

  if(flg_cr)
  {
    bfr[len]='\r';
    bfr[len+1]='\n';
    bfr[len+2]='\0';
  }
  else
    bfr[len]='\0';
}

/*
 * Returns the length of a string
 */

int v2_strlen(unsigned char *bfr)
{
	int to_ret;

	for(to_ret=0;bfr[to_ret];to_ret++)
		;

	return to_ret;
}

int CheckModuleInstallation( const unsigned char* modul_name)
{
	int buf;
	asm("
		mov $16, %%al;	//Get Modul Info
		mov %1, %%edi
		int $0x20

		mov %%edi, %0
		"
		: "=m" (buf)
		: "m" (modul_name)
		: "eax", "edi"
	);
	return buf;
}