/* gccmod.c */

/*
 * 22/12/1999 Joost Faassen, Carlo Prelz
 * Copyright (C) V2_Lab Rotterdam
 *
 * The example GCC module for V2_OS
 *
 * $Log: gccmod.c,v $
 * Revision 1.1  1999/12/23 10:16:20  karl
 * Initial revision
 *
 */

/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * For modules include v2os_mod
 */

#include "v2os_mod.h"

/*
 * Define the number of services that you want to provide.
 * Both declarations are necessary
 */

#define N_SERVICES 3
unsigned long n_local_services=N_SERVICES;

/*
 * These routines must be defined in your module. The first is run when
 * V2OS is loading your module. The second one is currently not used.
 */

void mod_load(void);
void mod_unload(void);

/*
 * Here I define the routines for the three services I provide. The name is up to you
 */

void mod_service1(void);
void mod_service2(void);
void mod_service3(void);

/*
 * The declaration of these arrays is required.
 * BASIC_SERVICES (16): the services of standard V2_OS modules. We must NOT
 * use these numbers.
 * LIB_SERVICES (currently 2): the services declared in modlib.c. We also ignore them.
 * N_SERVICES: our services.
 */

voidcall service_calls[BASIC_SERVICES+LIB_SERVICES+N_SERVICES];
service_stc services[LIB_SERVICES+N_SERVICES];

/*
 * The module name. Can be seen from the "modlist" command
 * Any length
 */

static const unsigned char mod_name[]="gccmod module";

/*
 * The module description. Can be seen from the module screen after loading
 * Any length
 */

static const unsigned char mod_desc[]="Test module for gcc/linux compiling";

/*
 * The module copyright. Can be seen from the module screen after loading
 * Any length
 */

static const unsigned char mod_copyright[]="Copyright (C) 1999 V2 Lab";

/*
 * IMPORTANT: Beginning of section HDR.
 * Don't change the order of the following lines
 *
 * Identifier. DON'T CHANGE!
 */

const char modhdr[4] __attribute__((section(".hdr")))="V2MD";

/*
 * ID of the module. This string is used when calling the module's services
 * from user-space programs
 * MUST BE 8 characters
 * and each module MUST have a different string if they have to work together
 */

const char modID[8] __attribute__((section(".hdr")))="V2 carlo";

/*
 * Module version, high/low. Set as preferred
 */

const char modver_lo __attribute__((section(".hdr")))='\0';
const char modver_hi __attribute__((section(".hdr")))='\1';

/*
 * OS version, high/low. LEAVE 0/0!!
 */

const char osver_lo __attribute__((section(".hdr")))='\0';
const char osver_hi __attribute__((section(".hdr")))='\0';

/*
 * The pointers to the above two subroutines. Don't touch
 */

voidcall modload_ptr __attribute__((section(".hdr")))=mod_load;
voidcall modunload_ptr __attribute__((section(".hdr")))=mod_unload;

/*
 * The pointer to the array of service calls. The array is defined above,
 * and will be filled up in mod_load
 */

voidcall *s_services __attribute__((section(".hdr")))=service_calls;

/*
 * Pointers to the above variable-length strings. Leave as is
 */

const char *modname __attribute__((section(".hdr")))=mod_name;
const char *moddesc __attribute__((section(".hdr")))=mod_desc;
const char *modcpyr __attribute__((section(".hdr")))=mod_copyright;

/*
 * END OF SECTION .hdr
 * You can change the order of lines now... :-)
 */


/*
 * Now the mod_run sub. It is run IF you decide to use the run command
 * instead of the loadmod command.
 * It should just warn the user to use loadmod.
 * But you can put whatever inside.
 */

void mod_run(void)
{
  puts("This is a \"module\" and cannot be run.\r\nPlease, use the 'loadmod' command instead!\r\r\n");
}

/*
 * What to do when loading the module
 * The messy part
 */

void mod_load(void)
{

/*
 * If the module provides a specific (named) service, here we announce it to the OS
 */
	
  announce("Tester");

/*
 * Fill up the services array
 * One of these arrays has to be filled for each service we provide.
 * We must inform the system of the subroutine to call, and of the (eventual) size of
 * the input or output parameter
 */

	services[0].in.datalen=0; /* No input data */
	services[0].out.datalen=0; /* No output data */
	services[0].call=mod_service1; /* What function do I call? */

	services[1].in.datalen=0; /* No input data */
	services[1].out.datalen=sizeof(unsigned long); /* Returns a long */
	services[1].call=mod_service2; /* What function do I call? */

	services[2].in.datalen=sizeof(unsigned short); /* Receives a short */
	services[2].out.datalen=sizeof(unsigned long); /* Returns a short */
	services[2].call=mod_service3; /* What function do I call? */

/*
 * You must then call this function. It does a series of mysterious
 * and dirty hacks to setup structures so that parameter passing is at
 * all possible.
 * You can look at the source in modlib.c, but I have to recover some energy before
 * I am even able to think about explaining the thing
 */

	fill_service_area();

/*
 * IMPORTANT!!
 * The last line of this function must be a call to macro RETF!!
 */
	
	RETF;
}

/*
 * not used but needs to be declared
 */

void mod_unload(void)
{
/*
 * IMPORTANT!!
 * The last line of this function must be a call to macro RETF!!
 */

	RETF;
}

/*
 * Now the functions that provide the services.
 * They MUST be of the format
 * void func(void);
 * parameter passing happens within, and eventual communication with external functions
 * has to be made with global static variables.
 */

/*
 * The first function has no parameters.
 * We just print out a confirmation string
 */

void mod_service1(void)
{
  puts("Inside module: service 1 has been called!\r\n");

/*
 * Functions that have no return parameters
 * MUST end with the RETF macro
 */

	RETF;
}

/*
 * The second function returns a long parameter to the caller.
 * The value is increased at each call
 */

void mod_service2(void)
{
	static unsigned long to_ret=0;
	int edi;

	to_ret++;

  puts("Inside module: service 2 is returning ");
	hexprintl(to_ret);
  puts("\r\n");

/*
 * The following call fills up the memory array with what is needed.
 * Parameters are:
 * o) The service name (progressive - we are service #1)
 * o) A flag: 0=users' services, 1=lib services. You will always have 0
 * o) A pointer to the memory area that contains the resulting data to pass over
 *    to the caller.
 *
 * The call returns the value that register edi has to be set to right before
 * returning
 */
	
	edi=ret_from_service(1,0,&to_ret);

/*
 * The call has to be closed with the RETF_EDI macro, passing the edi parameter
 * tha we obtained from ret_from_service
 */
	
	RETF_EDI(edi);
}

/*
 * The third function receives a short parameter, and returns a short to the caller.
 * We return a value that corresponds to the passed value plus an internal value that is
 * incremented each time
 */

void mod_service3(void)
{
	static unsigned short to_ret=0;
	static unsigned short tmp;
	int edi;

	to_ret++;

/*
 * The value that the calling program has passed to us
 * can be found in services[n_service].in.data;
 */
	
	tmp=to_ret+*(unsigned short *)services[2].in.data;

  puts("Inside module: service 3 is returning ");
	hexprints(tmp);
  puts("\r\n");

/*
 * The following call fills up the memory array with what is needed.
 * Parameters are:
 * o) The service name (progressive - we are service #2)
 * o) A flag: 0=users' services, 1=lib services. You will always have 0
 * o) A pointer to the memory area that contains the resulting data to pass over
 *    to the caller.
 *
 * The call returns the value that register edi has to be set to right before
 * returning
 */
	
	edi=ret_from_service(2,0,&tmp);

/*
 * The call has to be closed with the RETF_EDI macro, passing the edi parameter
 * tha we obtained from ret_from_service
 */
	
	RETF_EDI(edi);
}



