/* lib.c */

/*
 * 10/12/1999 Joost Faassen, Carlo Prelz
 * Copyright (C) V2_Lab Rotterdam
 *
 * Library of standard functions for developing modules and applications
 * for V2_OS in C/Linux
 *
 * $Log: lib.c,v $
 * Revision 1.2  1999/12/23 09:55:48  karl
 * After adapting to mod use
 *
 * Revision 1.1  1999/12/10 17:07:57  karl
 * Initial revision
 *
 */

/* This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "v2os.h"

int _start=0x100; /* complains if not defined */

/*
 * When arrays are passed to lib routines, they seem to have to be declared static and out
 * of subroutines
 */

unsigned char lib_in_bfr[256];
unsigned char lib_out_bfr[256];
static unsigned char puts_bfr[256];

#if 0
void printptrs(void)
{
	term_puts(SCR_DEBUG,"allbeg:");
	term_hexprintl(SCR_DEBUG,(unsigned long)&_allbeg);
	term_puts(SCR_DEBUG,"\r\netext:");
	term_hexprintl(SCR_DEBUG,(unsigned long)&_etext);
	term_puts(SCR_DEBUG,"\r\nebss1:");
	term_hexprintl(SCR_DEBUG,(unsigned long)&_ebss1);
	term_puts(SCR_DEBUG,"\r\nebss2:");
	term_hexprintl(SCR_DEBUG,(unsigned long)&_ebss2);
	term_puts(SCR_DEBUG,"\r\nedata:");
	term_hexprintl(SCR_DEBUG,(unsigned long)&_edata);
	term_puts(SCR_DEBUG,"\r\nerodata:");
	term_hexprintl(SCR_DEBUG,(unsigned long)&_erodata);
	term_puts(SCR_DEBUG,"\r\n");
}
#endif

/*
 * Test routine - calls System32 service #2 (NULL function)
 */

inline void nullf(void)
{
  asm volatile (
		"mov $2,%%eax;int $0x20"
		: 
		: 
		: "eax"
		);
}

/*
 * This one simply inserts a long return assembly instruction
 */

inline void retf(void)
{
	asm("lret");
}

/*
 * Write directly to memory
 */

inline void pokeb(unsigned short Seg, unsigned long Off, unsigned char Data)
{
  __asm__ __volatile__ (
    "movw %w0,%%fs \n.byte 0x64 \nmovb %w1,(%k2)"
    :
    : "rm" (Seg), "ri" (Data), "r" (Off));
}

inline void pokel(unsigned short Seg, unsigned long Off, unsigned long Data)
{
  __asm__ __volatile__ (
    "movw %w0,%%fs; .byte 0x64; movl %1,(%k2)"
    :
    : "rm" (Seg), "ri" (Data), "r" (Off));
}

/*
 * and read from it
 */

inline unsigned long peekl(unsigned short Seg, unsigned long Off)
{
	unsigned long to_ret;
	
  __asm__ __volatile__ (
    "movw %w1,%%fs; .byte 0x64; movl (%k2),%0"
    : "=ri" (to_ret)
    : "rm" (Seg),  "r" (Off));

	return to_ret;
}

/*
 * Write a string to the main terminal. User should add \r\n
 */

inline void puts(unsigned char *msg)
{
  int i;

  for(i=0;i<255 && msg[i]!='\0';i++)
    puts_bfr[i]=msg[i];
  puts_bfr[i]='\0';
  
  asm("mov $4,%%eax;mov %0,%%edi;int $0x20"
      : /* no outputs */
      : "m" (puts_bfr)
			: "eax", "edi"
      );
}

/*
 * Write a string to a given screen. User should add \r\n
 * <screen> should be set to one of the predefined variables
 * in v2os.h (SCR_TERMINAL, etc)
 */

inline void term_puts(unsigned char screen,unsigned char *msg)
{
  int i;

  for(i=0;i<255 && msg[i]!='\0';i++)
    puts_bfr[i]=msg[i];
  puts_bfr[i]='\0';
  
  asm("mov $38,%%eax;mov %0,%%edx;mov %1,%%edi;int $0x20"
      : /* no outputs */
      : "m" (screen), "m" (puts_bfr)
			: "eax", "edx", "edi"
      );
}

/*
 * Pauses for given amount of 25ths of second
 */

void delay(unsigned long sec_25ths)
{
  asm("mov $26,%%eax;mov %0,%%edi;int $0x20"
      : /* no outputs */
      : "m" (sec_25ths)
			: "eax", "edi"
      );
}

/*
 * Print a byte, a short or a long to main terminal in hex
 */

void hexprintb(unsigned char val)
{
  asm("mov $0x105,%%eax;mov %0,%%ebx;int $0x20"
      : /* no outputs */
      : "m" (val)
			: "eax", "ebx"
      );
}
void hexprints(unsigned short val)
{
  asm("mov $0x205,%%eax;mov %0,%%ebx;int $0x20"
      : /* no outputs */
      : "m" (val)
			: "eax", "ebx"
      );
}
void hexprintl(unsigned long val)
{
  asm("mov $0x405,%%eax;mov %0,%%ebx;int $0x20"
      : /* no outputs */
      : "m" (val)
			: "eax", "ebx"
      );
}

/*
 * Print a byte, a short or a long to a screen in hex. <screen>
 * is set as in term_puts above
 */

void term_hexprintb(unsigned char screen,unsigned char val)
{
  asm("mov $0x127,%%eax;mov %0,%%edx;mov %1,%%ebx;int $0x20"
      : /* no outputs */
      : "m" (screen), "m" (val)
			: "eax", "ebx"
      );
}
void term_hexprints(unsigned char screen,unsigned short val)
{
  asm("mov $0x227,%%eax;mov %0,%%edx;mov %1,%%ebx;int $0x20"
      : /* no outputs */
      : "m" (screen), "m" (val)
			: "eax", "ebx"
      );
}
void term_hexprintl(unsigned char screen,unsigned long val)
{
  asm("mov $0x427,%%eax;mov %0,%%edx;mov %1,%%ebx;int $0x20"
      : /* no outputs */
      : "m" (screen), "m" (val)
			: "eax", "ebx"
      );
}

/*
 * Waits for a keypress. Returns the character
 */

unsigned char getkey(void)
{
	unsigned long l;
	
  asm("mov $23,%%eax;int $0x20;mov $29,%%eax;int $0x20;mov %%eax,%0"
			: "=m" (l)
			:
			: "eax"
			);

	return (unsigned char)l;
}

/*
 * Reads a string from terminal. Must pass buffer with enough space
 */

void gets(unsigned char *bfr)
{
	int i;	

  asm("mov $40,%%eax;mov %0,%%edi;int $0x20"
      : /* no outputs */
      : "m" (lib_out_bfr)
			: "eax", "edi"
      );


	for(i=0;i<255 && lib_out_bfr[i];i++)
		bfr[i]=lib_out_bfr[i];
	
	bfr[i]='\0';
}

/*
 * Executes a V2_OS shell command
 */

void shell_command(unsigned char *cmd)
{
	int i;

  for(i=0;i<255 && cmd[i]!='\0';i++)
    puts_bfr[i]=cmd[i];
  puts_bfr[i]='\0';
	
  asm("mov $25,%%eax;mov %0,%%edi;int $0x20"
      : /* no outputs */
      : "m" (puts_bfr)
			: "eax", "edi"
      );
}

/*
 * Obtain the address for the input data area for a service
 */

void *service_input_data(unsigned char *modname,int req_service)
{
	static unsigned long *service=NULL;

	if(service==NULL)
	{
		service=abs_to_relative(*((unsigned long *)modservice(modname,SVC_GET_PROTO_INPUT_ADDR)));
		
		if(service==NULL)
		{
			term_puts(SCR_DEBUG,"Module ");
			term_puts(SCR_DEBUG,modname);
			term_puts(SCR_DEBUG," does not provide SVC_GET_PROTO_INPUT_ADDR!\r\n");

			return NULL;
		}
	}

	*service=req_service;	

	return abs_to_relative(*((unsigned long *)modservice(modname,SVC_GET_INPUT_ADDR)));
}

/*
 * Call a module's service
 */

void *modservice(unsigned char *modname,int service)
{
	static int i,par;
	static unsigned long to_ret;

	par=21|(service<<8);
	
  for(i=0;i<8;i++)
    lib_in_bfr[i]=modname[i];
  lib_in_bfr[i]='\0';

  asm("mov %1,%%eax;mov %2,%%edi;int $0x20;mov %%edi,%0"
      : "=m" (to_ret)
      : "m" (par), "m" (lib_in_bfr)
			: "eax", "edi"
      );
	
	if(to_ret)
	{
		open_memory();
		to_ret-=whats_my_address();
	}

	return (void *)to_ret;
}

/*
 * Sets a flag that allows you to access to the whole memory
 */

void open_memory(void)
{
	static unsigned char alreadycalled=0;
	
  if(!alreadycalled)
	{
		asm("mov $44,%%eax;mov %%ds,%%bx;mov $0xfffff000,%%edi;int $0x20"
				:
				:
				: "eax", "bx", "edi"
				);
		alreadycalled=1;
	}
}

/*
 * Allocates an area of memory from V2_OS. Returns a pointer relative to
 * the caller's memory, and the absolute address in abs_address
 * mem_pos is MEM_LOW if obtaining memory from low addresses, and MEM_HIGH if from top
 * of available memory
 */

void *malloc(int size,unsigned char mem_pos,unsigned short *ret_sel,unsigned long *abs_address)
{
	unsigned long ret_size,my_address;
	unsigned short selector;
	unsigned char service;

	if(mem_pos!=MEM_LOW && mem_pos!=MEM_HIGH)
	{
		term_puts(SCR_DEBUG,"Error! malloc mem_pos = ");
		term_hexprintb(SCR_DEBUG,mem_pos);
		term_puts(SCR_DEBUG,"!\r\n");

		return NULL;
	}
		
	service=(mem_pos==MEM_LOW) ? 14 : 45;

	open_memory();
	
  asm("mov %4,%%eax;mov %3,%%ebx;mov $0,%%edx;int $0x20;mov %%ebx,%1;mov %%cx,%2;mov %%edi,%0"
      : "=m" (my_address), "=m" (ret_size), "=m" (selector)
      : "m" (size), "m" (service)
			: "eax", "ebx", "edx", "cx", "edi"
      );

	*abs_address=my_address;
	*ret_sel=selector;

	return abs_to_relative(*abs_address);
}

/*
 * Converts an absolute address to an address pointer relative to my memory area
 */

void *abs_to_relative(unsigned long abs_address)
{
	unsigned long my_address=whats_my_address();
	
	return (void *)(abs_address-my_address);
}	

/*
 * Returns the start of my address space
 */

unsigned long whats_my_address(void)
{
	static unsigned long to_ret=0;

	if(!to_ret)
	{
		asm("mov $43,%%eax;mov %%cs,%%bx;int $0x20;mov %%edi,%0"
				: "=m" (to_ret)
				:
				: "eax", "bx", "edi"
				);
	}
	
	return to_ret;
}

void v2_memcpy(void *from,void *to,int len)
{
	asm("push %%es;push %%ds;pop %%es;mov %0,%%esi;mov %1,%%edi;mov %2,%%ecx;.byte 0xf3;.byte 0xa4;pop %%es"
			:
			: "m" (from), "m" (to), "m" (len)
			: "esi", "edi", "ecx"
			);
	
}

/*
 * C routines...
 *
 * Convert from number to string.
 * val: input value
 * bfr: resulting string. Buffer is passed from caller.
 * len: length of the resulting string. If number is smaller,
 * string is completed with 0's,
 */

void num_to_string(int val,unsigned char *bfr,int len,unsigned char flg_cr)
{
  int i;
  
  for(i=len-1;i>=0;i--)
  {
    bfr[i]='0'+(val%10);
    val/=10;
  }

  if(flg_cr)
  {
    bfr[len]='\r';
    bfr[len+1]='\n';
    bfr[len+2]='\0';
  }
  else
    bfr[len]='\0';
}

/*
 * Returns the length of a string
 */

int v2_strlen(unsigned char *bfr)
{
	int to_ret;

	for(to_ret=0;bfr[to_ret];to_ret++)
		;

	return to_ret;
}

/*
 * A way to print a pointer's value
 */

void print_abs_address(unsigned char *name,void *ptr)
{
	static unsigned long abs_address;
	
	abs_address=whats_my_address()+(unsigned long)ptr;
	
	term_puts(SCR_DEBUG,name);
	term_puts(SCR_DEBUG," is <");
	term_hexprintl(SCR_DEBUG,abs_address);	
	term_puts(SCR_DEBUG,"> (abs)\r\n");
}

void print_address(unsigned char *name,void *ptr)
{
	term_puts(SCR_DEBUG,name);
	term_puts(SCR_DEBUG," is <");
	term_hexprintl(SCR_DEBUG,(unsigned long)ptr);	
	term_puts(SCR_DEBUG,">\r\n");
}
	
