/* modlib.c */

/*
 * 10/12/1999 Joost Faassen, Carlo Prelz
 * Copyright (C) V2_Lab Rotterdam
 *
 * Library of standard functions for developing *modules*
 * for V2_OS in C/Linux
 *
 * $Log: modlib.c,v $
 * Revision 1.1  1999/12/23 09:54:22  karl
 * Initial revision
 *
 */

/* This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "v2os_mod.h"

/*
 * You must declare this in your module source
 */

extern const char modID[];

void get_proto_input_addr(void);
void get_input_addr(void);
void update_service_area(service_data_stc *svc_data);

/*
 * Library services. They are needed to be able to exchange parameters
 * to/from user services.
 *
 * The first one returns the address of the input area of the second one.
 * The second one returns the address of the input area of a given
 * user service
 */

service_stc lib_services[LIB_SERVICES]=
{
	{
		{NULL,0L,0L},
		{NULL,sizeof(unsigned long),0L},
		get_proto_input_addr
	},
	{
		{NULL,sizeof(unsigned long),0L},
		{NULL,sizeof(unsigned long),0L},
		get_input_addr
	}
};

/*
 * Announces to V2_OS that
 * we are providing a specific type of service
 */

void announce(unsigned char *servtype)
{
  int i;
  
  for(i=0;i<15 && servtype[i]!='\0';i++)
    lib_in_bfr[i]=servtype[i];
  lib_in_bfr[i]='\0';
  
  asm("mov $17,%%eax;mov %0,%%edi;mov %1,%%esi;int $0x20"
      : /* no outputs */
      : "m" (modID), "m"(lib_in_bfr)
			: "eax", "edi", "edi"
      );
}

/*
 * The routine that fills the service area data arrays
 * To be called once in load_mod (see example)
 */

void fill_service_area(void)
{
	int i;
	
	for(i=0;i<n_local_services;i++)
	{
		update_service_area(&services[i].in);
		update_service_area(&services[i].out);
	}
	for(i=0;i<LIB_SERVICES;i++)
	{
		update_service_area(&lib_services[i].in);
		update_service_area(&lib_services[i].out);
	}	
		
/*
 * fill up the call table
 */

	for(i=0;i<BASIC_SERVICES;i++)
		service_calls[i]=(voidcall)0;
	for(i=0;i<LIB_SERVICES;i++)
		service_calls[BASIC_SERVICES+i]=lib_services[i].call;
	for(i=0;i<n_local_services;i++)
		service_calls[BASIC_SERVICES+LIB_SERVICES+i]=services[i].call;
}

/*
 * This one makes sure that the output data from a service is stored in the correct
 * memory area. The pointer to return to the caller program is returned
 */

unsigned long ret_from_service(int service,unsigned char lib_flg,void *data)
{
	static service_data_stc *svcdata;

	if(!lib_flg)
		svcdata=&services[service].out;
	else
		svcdata=&lib_services[service].out;
	
	v2_memcpy(data,svcdata->data,svcdata->datalen);
	
	return svcdata->abs_address;
}

/*
 * The following two routines represent the operative part of the library services.
 * The first one returns the address of ine input data area of the second one
 * (requires no input data) ...
 */

void get_proto_input_addr(void)
{
	int edi;
	
	edi=ret_from_service(SVC_GET_PROTO_INPUT_ADDR-BASIC_SERVICES,1,
											 &(lib_services[SVC_GET_INPUT_ADDR-BASIC_SERVICES].in.abs_address));
	RETF_EDI(edi);
}

/*
 * ... And the second one returns the address of the input area of a given user service,
 * given its number
 */

void get_input_addr(void)
{
	unsigned long service=*(unsigned long *)(lib_services[SVC_GET_INPUT_ADDR-BASIC_SERVICES].in.data),
		edi;
	static unsigned long abs_address;

	service-=BASIC_SERVICES+LIB_SERVICES;
	
	if(service>=0 && service<n_local_services)
		abs_address=services[service].in.abs_address;
	else
		abs_address=0;

	edi=ret_from_service(SVC_GET_INPUT_ADDR-BASIC_SERVICES,1,&abs_address);
	
	RETF_EDI(edi);	
}

/*
 * This sub allocates the necessary space for either the input or the output
 * data spaces of services. The absolute address is also saved for
 * further reference
 */

void update_service_area(service_data_stc *svc_data)
{
	static unsigned short selector;
	static unsigned long abs_address;
	
	if(svc_data->datalen)
	{
		svc_data->data=malloc(svc_data->datalen,MEM_HIGH,&selector,&abs_address);
		svc_data->abs_address=abs_address;
	}
	else
	{
		svc_data->data=NULL;
		svc_data->abs_address=0;
	}
}
	
	
