;-------------------------------------------------------------------------
; PlayWave
; --------
; Demonstrates the use of the PlayStream service 15h of the sound module.
; It needs a wave file named 'test.wav'.
;-------------------------------------------------------------------------


[bits 32]
org 100h

	mov	al, 10h
	mov	edi, moduleID
	int	20h			; Module lookup
	cmp	edi, 0			; Returns 0 if module not loaded
	je	fail

	mov	al, 15h			; call service 10h (init check)
	mov	ah, 10h			; returns 0 if sound module
	mov	edi, moduleID		; was init'd ok
	int	20h
	cmp	al,0
	jne	LoadFail

	mov	al,04h
	mov	edi,OKMsg
	int	20h

	mov	ax,ds
	mov	word [DataSel],ax

	call	LoadWaveFile		; load test.wav
	cmp	al,0			; errors?
	jne	NotOK
	call	ShowWaveInfo		; display wave info
	call	PlayStartSound		; start playing

PlayWaitLoop:
	cmp	byte [streamFlg],0	; wait until we're finished
	je	PlayWaitLoop

	mov	al, 15			; free wavedata
	mov	bx, [WaveSel]
	int	20h
	
	retf
NotOK:
	mov	al,04h
	mov	edi,NotOKMsg
	int	20h
	retf	

LoadFail:
	mov	al,04h
	mov	edi,LoadFailMsg
	int	20h
	retf
fail:
	mov	al, 04h
	mov	edi, failMsg
	int	20h			; Print string
	retf


;------------------------------------------------------------
; LoadWaveFile
; --------------
; Allocates a buffer into memory and loads a wav file
; from floppy to this buffer
;------------------------------------------------------------


LoadWaveFile:
	mov	al,0Ah			; GetFileInfo
	mov	edi,wavefile		; offset to filename
	int	20h
	cmp	al,0			; error
	je	LWF_FileOk		; no, continue 

	ret				; else return

LWF_FileOk:
	push	eax
	push	ecx
	push	edi
	
	shl	ecx,9
	mov	dword [WaveFileSize],ecx

	mov	al,04h			; print loading text
	mov	edi,WaveLoad
	int	20h

	pop	edi
	pop	ecx
	pop	eax
	
; Load File

	push	ebx
	push	ecx
	push	edx
	
	mov	ebx,ecx
	inc	ebx
	shl	ebx,9
	mov	dword [streamEnd],ebx
	mov	al,0Eh			; request mem+selector
	mov	dx,0100h		; 32bit data
	int	20h
	mov	word [WaveSel],cx
	
	pop	edx
	pop	ecx
	pop	ebx

	; edi = Abs. address of mem
	; ebx = start sector
	; ecx = number of sectors
	; dl  = disk

	push	ds
	push	es
	push	gs
	pop	ds			; ds:esi - source
	push	gs
	pop	es			; es:edi - target

LWF_LoadLoop:				; load the file sectors
	push	ecx
	push	ebx
	mov	al,06h			; system32 - read sectors
	mov	ecx,1			; read 1 sector	
	int	20h
	mov	si,[gs:600H+82]
	movzx	esi, si
	shl	esi, 4			; DISKBUFFER ADDR
	shl	ecx, 9			; *512
	rep	movsb			; copy sector to memory
	pop	ebx
	pop	ecx
	inc	ebx			; next sector
	loop	LWF_LoadLoop		; and read it

	pop	es
	pop	ds	

	mov	al,04h
	mov	edi,WaveLdOk
	int	20h

	mov	al,0
	ret


;------------------------------------------------------------
; PlayStartSound
; --------------
; This function starts the playback of the wavefile by
; calling service 15h of the sound module.
; The service is called with :
; ES:EDX - pointer to callback routine (see below)
; EBX	 - samplerate
;------------------------------------------------------------

PlayStartSound:
	push	eax
	push	ebx
	push	edx
	push	edi
	push	es

	mov	dword [streamPos],0	; reset stream to start
	mov	byte  [streamFlg],0	; indicate we're still playing

	mov	es,[WaveSel]
	xor	ebx,ebx	
	mov	bx, word [es:WAV_SAMPLERATE]
	mov	cl,8
	mov	ch,[stereo]

	push	cs
	pop	es
	mov	edx, StreamCallBack	; es:edx address of callback function
	mov	al, 15h
	mov	ah, 15h			; call service 05h of v2sound (PlayStream)
	mov	edi, moduleID
	int	20h			; 

	pop	es
	pop	edi
	pop	edx
	pop	ebx
	pop	eax		
	ret

;--------------------------------------------------------------------
; StreamCallBack
; --------------
; This function is called by the sound module whenever it needs
; more data to play.
; IN	: ECX - number of bytes needed
;	  EDI - Linear address to copy data to
; OUT	: EAX - number of bytes copied ( 0 = stop stream )
;
; If there are less bytes available than requested, it's preferred
; that the rest of the buffer is filled with silence bytes (0x80).
; This is to avoid noise and clicks in playback.
; It's not done here, but it should be.. ;)
;--------------------------------------------------------------------
StreamCallBack:
	push	ebx
	push	ecx
	push	edx
	push	edi
	push	esi
	push	ds
	push	es

	mov	bx,[cs:DataSel]
	mov	ds,bx

	mov	esi,dword [streamPos]
	add	esi,ecx
	cmp	esi,dword [streamEnd]		; do we have enough bytes left?
	jbe	SCB_CopyECXBytes		; yes, copy requested number of bytes
	
	mov	esi,dword [streamEnd]		; else, see how much we have left
	sub	esi,dword [streamPos]
	js	SCB_StopStream			; pos>end, then stop (this really shouldn't happen..)
	mov	ecx,esi
	
	jcxz	SCB_StopStream			; zero bytes left to play, also stop

	jmp	SCB_CopyECXBytes

SCB_StopStream:
	mov	eax,0				; no more bytes left
	mov	byte [streamFlg],1		; so we can stop the stream
	jmp	SCB_End
	
SCB_CopyECXBytes:
	mov	esi,dword [streamPos]
	add	dword [streamPos],ecx		; update position in wave
	push	gs
	pop	es				; target in es:edi
	mov	ds,[cs:WaveSel]			; source in ds:esi
					
	rep	movsb				; copy ecx bytes

SCB_End:
	pop	es
	pop	ds
	pop	esi
	pop	edi
	pop	edx
	pop	ecx
	pop	ebx
	retf



;-----------------------------------------------------------------
; ShowWaveInfo
; ------------
; Prints some information on the wav file
;-----------------------------------------------------------------


WaveInfo	DB	'File info',13,10
		DB	'------------------------',13,10,0
WaveName	DB	'Filename      : ',0
WaveSize	DB	'Size          : ',0
WaveFileSize	DD	0
WaveBitSize	DB	'Sample size   : ',0
WaveS16		DB	'16 bit',13,10,0
WaveS8		DB	'8 bit',13,10,0
WaveAuType	DB	'Audio format  : ',0
WaveAuPCM	DB	'PCM',13,10,0
WaveAuOther	DB	'Unknown',13,10,0
WaveChannels	DB	'Channels      : ',0
WaveChMono	DB	'1 (mono)',13,10,0
WaveChStereo	DB	'2 (stereo)',13,10,0

; offsets of various data in wav header

WAV_FORMAT	equ	0x14
WAV_CHANNELS	equ	0x16
WAV_SAMPLERATE	equ	0x18
WAV_BYTERATE	equ	0x1C
WAV_BITSIZE	equ	0x22


ShowWaveInfo:
	push	eax
	push	edi
	push	es

	mov	es,[WaveSel]

	mov	al,04h
	mov	edi,WaveInfo
	int	20h

	mov	al,04h
	mov	edi,WaveName
	int	20h
	mov	edi,wavefile
	int	20h
	mov	edi,CRLF
	int	20h
	
	mov	edi,WaveSize
	int	20h
	mov	ebx,[WaveFileSize]
	mov	ax,0405h
	int	20h
	mov	al,04h
	mov	edi,CRLF
	int	20h

	mov	al,04h				; print audio format
	mov	edi,WaveAuType
	int	20h
	mov	ax,[es:WAV_FORMAT]
	cmp	ax,1
	jne	SWI_FMTUNK
	mov	al,04h
	mov	edi,WaveAuPCM
	int	20h
	jmp	SWI_PRBITS
SWI_FMTUNK:
	mov	al,04h
	mov	edi,WaveAuOther
	int	20h

SWI_PRBITS:
	mov	al,04h				; print bitsize
	mov	edi,WaveBitSize
	int	20h
	mov	ax,[es:WAV_BITSIZE]
	cmp	ax,8
	jne	SWI_16bit
	mov	al,04h
	mov	edi,WaveS8
	int	20h
	jmp	SWI_PRCHANNELS
SWI_16bit:
	mov	al,04h
	mov	edi,WaveS16
	int	20h

SWI_PRCHANNELS:
	mov	al,04h				; print channels mono/stereo
	mov	edi,WaveChannels
	int	20h
	mov	ax,[es:WAV_CHANNELS]
	cmp	ax,1
	jne	SWI_CHSTEREO
	mov	al,04h
	mov	edi,WaveChMono
	int	20h
	mov	byte [stereo],0
	jmp	SWI_END
SWI_CHSTEREO:
	mov	al,04h
	mov	edi,WaveChStereo
	int	20h	
	mov	byte [stereo],2
SWI_END:
	pop	es
	pop	edi
	pop	eax	
	ret	



; vars
moduleID	DB	'V2_SOUND',0
failMsg		DB	'The required module: v2sound.v2m is not loaded!', 0Dh, 0Ah, 0
NotOKMsg	DB	'The v2sound module wasn',39,'t initialized properly!', 13,10,0
LoadFailMsg	DB	'Can',39,'t load file test.wav',13,10,0
OKMsg		DB	'The v2sound module is ready to go!',13,10,0
CBMsg		DB	'C',0
WaveLoad	DB	'Loading, please wait...',0
WaveLdOk	DB	'Ok',13,10,13,10,0
stereo		DB	0
CRLF		DB	13,10,0
wavefile	DB	'test.wav',0
streamPos	DD	0x2C		; current position ( skip the wave header)
streamEnd	DD	0		; size of wave file
streamFlg	DB	0		; still playing flag
DataSel		DW	0
WaveSel		DW	0