;=============================================================
; SoundBlaster DSP routines (not complete)
;
; Author   : Bjorge Dijkstra (bjorge@gmx.net)
; Date     : 20-03-2000
;
; last rev.: 23-03-2000
;
;
;=============================================================


;=============================================================
;
;  DSP Routines:
;  --------------------------------------------------------
; xDSPInit			- Initialize everything
; *DSPWrite			- Write byte to DSP
; *DSPRead			- Read byte from DSP
; *DSPReset			- Reset DSP
; *DSPDetectBase		- Find DSP Baseport
; *DSPGetVersion		- Get DSP version
; *DSPSetSpeaker		- Turns speakers on/off
; *DSPSetSampleRate		- Sets sample rate
; *DSPDMA8DAC			- 8bit DMA playback
; *DSPDMA16DAC			- 16bit DMA playback
; *DSPDMARepeat			- Repeat last DMA cmd
; *DSPDMAExit			- Stop DMA
;
;=============================================================



;=============================================================
; CONSTS
;=============================================================

	SB1X	equ	1
	SB2X	equ	2
	SBPRO	equ	3
	SB16	equ	4
	
;=============================================================
; DATA
;=============================================================

	DSPVMajor	DB	0
	DSPVMinor	DB	0
	DSPBase		DW	0
	DSPLoDMA	DB	0
	DSPHiDMA	DB	0
	DSPIRQ		DB	0
	DSPChannels	DB	0
	DSPBits		DB	8
	DSPDMAAutoInit	DB	0
	DSPDMABlockSize DW	0

;=============================================================
; CODE
;=============================================================


;=============================================================
; DSPInit
; --------
; IN :
;       DX = DSP base port
;       AL = LoDMA
;       AH = HiDMA
;       BL = IRQ
; OUT:
;	AL == 0 on success
;=============================================================
DSPInit:
	mov	[DSPBase],dx
	mov	[DSPLoDMA],al
	mov	[DSPHiDMA],ah
	mov	[DSPIRQ],bl
	mov	al,0
	ret


;=============================================================
; DSPWrite
; --------
; IN :
;       AL = Data byte
;=============================================================
DSPWrite:
        push	edx         
        push	eax
	mov	dx,[DSPBase]
        add	dx,0ch			; select write data port
WriteDSP_Busy:
        in	al,dx              	; get write status
        and	al,80h              	; ready to write ?
        jnz	WriteDSP_Busy       	; no check status again
        pop	eax 
        out	dx,al               	; write data byte
        pop	edx
        ret                      	; return to caller


;=============================================================
; DSPRead
; --------
; IN :  nothing
; OUT:
;       AL - Data byte
;=============================================================
DSPRead:
        push	edx
	mov	dx,[DSPBase]
        add	dx,0eh              	; select data ready status port
ReadDSP_Busy:
        in	al,dx               	; get status
        and	al,80h
        jz 	ReadDSP_Busy
        sub 	dx,4                	; select data read port
        in   	al,dx               	; read byte
        pop  	edx
        ret     	            	; return to caller



;=============================================================
; DSPReset
; --------
; IN :  nothing
; OUT:
;       carry set    = failed
;             clear  = success              
;=============================================================
DSPReset:
        push 	ax                  	; save used registers
        push 	cx
        push 	dx

	mov	dx,[DSPBase]
        add  	dx,6                	; select DSP reset port (DX = base+06h)
        mov  	al,1                	;
        out  	dx,al               	; send AL (=1) to reset port
        mov  	eax,0FFFh
DSPReset_delay:                  	; simple delay loop
        dec  	al                  	;
        jnz  	DSPReset_delay      	; decrease AL until zero
        out  	dx,al               	; send AL (=0) to reset port
        mov  	cx, 0FFFh
        add  	dl,8                	; select DSP data ready port (dx = base+0eh)
DSPReset_Status: 
        in   	al,dx               	; 
        or   	al,al               	; data ready to be read?
        jns  	DSPReset_Retry      	; if not, try again
        sub  	dl,4                	; select DSP data port (dx = base + 0ah) 
        in   	al,dx               	; read DSP data
        cmp  	al,0aah             	; is al expected value?
        je   	DSPReset_Ok         	; yes, then reset was successfull
        add  	dl,4
DSPReset_Retry:
        loop 	DSPReset_Status     	; try to read status value again
        stc                      	; all attempts failed,  set carry
        jmp  	DSPReset_exit
DSPReset_Ok:
        clc                      	; success, clear carry
DSPReset_exit:
        pop  	dx
        pop  	cx                  	; restore saved registers
        pop  	ax 
        ret                      	; and return to caller



;=============================================================
; DSPDetectBase
; -------------
; OUT:
;       carry   =  clear, success
;                  set,   failed
;       DX      =  if success, DSP base port
;=============================================================  
DSPDetectBase:
        mov  	dx,0210h                ; First valid DSP port
DSPDetectBase_TryPort:        
	mov	[DSPBase],dx
        call 	DSPReset                ; try to reset DSP
        jnc  	DSPDetectBase_OK
        add  	dx,10h                  ; try next port
        cmp  	dx,0290h                ; tried all ports?
        je   	DSPDetectBase_Fail      
        jmp  	DSPDetectBase_TryPort 
DSPDetectBase_OK:
        clc                          	; clear carry
	ret
DSPDetectBase_Fail:
        stc                          	; set carry   
        ret                          	; return to caller   



;=============================================================
; DSPGetVersion
; -------------
; IN :
;	nothing
; OUT:
;       AH - Version Major
;       AL - Version Minor
;=============================================================
DSPGetVersion:	
	mov  	al, 0E1h		; command E1h = Get DSP Version
        call 	DSPWrite           	; send command
        call 	DSPRead            	; read Major
	mov	[DSPVMajor],al		;
        shl  	ax,8
        call 	DSPRead			; read Minor
	mov	[DSPVMinor],al		;
        ret


;=============================================================
; DSPSetSpeaker
; -------------
; IN :
;	AL - on/off ( 0=off, else on )
; OUT:  
;	nothing
;=============================================================
DSPSetSpeaker:
	push	ax
	or	al,al
	jnz	DSPSetSpeakerOn
	mov	al,0D3h
	call	DSPWrite
	pop	ax
	ret
DSPSetSpeakerOn:
	mov	al,0D1h
	call	DSPWrite
	pop	ax
	ret


;=============================================================
; DSPSetSampleRate
; ----------------
; IN :
;	BX - sample rate
; OUT:
;       AL - on/off ( 0=off, else on )
;=============================================================
DSPSetSampleRate:
	push	eax			; save used registers
	push	edx
	mov	dx,[DSPBase]
	cmp	byte [DSPVMajor],SB16
	jl	DSPSetSampleRateOldSB
	
        mov	al,41h			; Set Sample Rate DSP command
	call	DSPWrite		; 	
	mov	al,bh
	call	DSPWrite		; 
	mov	al,bl
	call	DSPWrite
	
	pop	edx			; restore registers
	pop	eax
	ret

DSPSetSampleRateOldSB:
        mov	al,40h			; Set Sample Rate DSP command
	call	DSPWrite		; 	

	push	ebx
        xor	edx,edx			; edx = 0
        mov	eax,1000000		; eax = 1.000.000
	div	ebx			; calc. eax = (edx:eax) div ebx
        neg	al			; al = 256 - al
        
	mov	dx,[DSPBase]
	call	DSPWrite		; Write TC (=AL) to DSP

	pop	ebx	
	pop	edx			; restore registers
	pop	eax
	ret


;=============================================================
; DSPDMADAC8
; ----------
; IN :
;	CX - block size
; OUT:
;	nothing
;=============================================================
DSPDMADAC8:
	mov	[DSPDMABlockSize],cx
	mov	byte [DSPBits],8
	cmp	byte [DSPVMajor],SB16
	jge	DSPDMADAC8_SB16		; SB16 and up
	cmp	byte [DSPVMajor],SB2X
	jge	DSPDMADAC8_SB2X		; SB2.x and SBPro

	mov	al,14h			; 8bit dma dac (no autoinit)
	call	DSPWrite
	mov	al,cl
	call	DSPWrite
	mov	al,ch
	call	DSPWrite
	mov	byte [DSPDMAAutoInit],0
	ret

DSPDMADAC8_SB2X:
	inc	cx
	shr	cx,1
	dec	cx
	mov	al,48h			; set dma block size
	call	DSPWrite
	mov	al,cl
	call	DSPWrite
	mov	al,ch
	call	DSPWrite
	mov	al,1Ch			; autoinit 8bit dma dac
	call	DSPWrite
	mov	byte [DSPDMAAutoInit],1
	ret
	
DSPDMADAC8_SB16:
	mov	al,0C6h
	call	DSPWrite
	mov	al,[DSPChannels]
	shl	al,4
	call	DSPWrite
	mov	al,cl
	call	DSPWrite
	mov	al,ch
	call	DSPWrite
	mov	byte [DSPDMAAutoInit],1
	ret

;=============================================================
; DSPDMADAC16
; -----------
; IN :
;	CX - block size
; OUT:
;	nothing
;=============================================================
DMA16Msg	db	'DMA SB16',13,10,0
DSPDMADAC16:
	mov	[DSPDMABlockSize],cx
	mov	byte [DSPBits],16
	cmp	byte [DSPVMajor],SB16
	jge	DSPDMADAC16_SB16		; SB16 and up
	ret

DSPDMADAC16_SB16:
	shr	ecx,1
	mov	al,0B6h
	call	DSPWrite
	mov	al,[DSPChannels]
	shl	al,4
	add	al,10h				; 16bit signed samples
	call	DSPWrite
	mov	al,cl
	call	DSPWrite
	mov	al,ch
	call	DSPWrite
	mov	byte [DSPDMAAutoInit],1
	ret


;=============================================================
; DSPDMARepeat
; ------------
; IN :
;	nothing
; OUT:
;	nothing
;=============================================================
DSPDMARepeat:
	cmp	byte [DSPVMajor],SB1X
	je	DSPDMARepeatSB1X
	ret	

DSPDMARepeatSB1X:
	push	eax
	push	ecx
	mov	cx,[DSPDMABlockSize]
	mov	al,14h			; 8bit dma dac (no autoinit)
	call	DSPWrite
	mov	al,cl
	call	DSPWrite
	mov	al,ch
	call	DSPWrite
	pop	ecx
	pop	eax
	ret
	
;=============================================================
; DSPDMAExit
; ------------
; IN :
;	nothing
; OUT:
;	nothing
;=============================================================
DSPDMAExit:
	cmp	byte [DSPDMAAutoInit],1
	je	DSPDMAExitAI
	cmp	byte [DSPBits],8
	jne	DSPDMAExitSB16
	mov	al,0D0h			; exit 8bit dma
	call	DSPWrite	
	ret
DSPDMAExitSB16
	mov	al,0D5h			; exit 16bit dma
	call	DSPWrite
	ret

DSPDMAExitAI:
	cmp	byte [DSPBits],8
	jne	DSPDMAExitAISB16
	mov	al,0D0h			; exit 8bit auto init dma
	call	DSPWrite
	mov	al,0DAh
	call	DSPWrite
	mov	al,0D0h
	call	DSPWrite	
	ret	
DSPDMAExitAISB16:
	mov	al,0D5h			; exit 16bit auto init dma
	call	DSPWrite
	mov	al,0D9h
	call	DSPWrite
	mov	al,0D5h
	call	DSPWrite	
	ret	
