.MODEL TINY
.386


code32  segment para public use32
code32  ends

code32  segment para public use32
        assume cs:code32, ds:code32
        ORG 100H 

SBPlay:

	jmp  start		; skip data and subroutines

include sweep.inc		; Sound data (8-bit 8000Hz WAV)
include sbdsp.asm		; SoundBlaster DSP routines
include dma.asm			; DMA transfer routines

; some constants
	PIC1	equ  020h	; First  PIC (handles IRQ 0-7)
	PIC2	equ  0A0h	; Second PIC (handles IRQ 8-15)
	EOI	equ  020h	; End-Of-Interrupt cmd


; data

        Welcome         DB 'SBPLAY 0.5 by Bjorge Dijkstra (bjorge@gmx.net)',13,10,13,10
        		DB 'This app should play an 8-bit sample on your SB using DMA',13,10
        Return          DB 13,10,0
	AskIRQ		DB 'Select your IRQ : ',13,10
			DB '0) IRQ  2',13,10
			DB '1) IRQ  3',13,10
			DB '2) IRQ  5',13,10
			DB '3) IRQ  7',13,10
			DB '4) IRQ 10',13,10
			DB 'Enter number    :',0
        AskDMA		DB 13,10,13,10,'Select your DMA : ',13,10
			DB '0) DMA  0',13,10
			DB '1) DMA  1',13,10
			DB '2) DMA  3',13,10
			DB 'Enter number    :',0
	SBNotFound	DB 13,10,'SB not found.. :(',13,10,0
 	SBOnPort	DB 13,10,'Using Port ',0
	SBOnIRQ		DB ', IRQ ',0
	SBOnDMA		DB ', DMA ',0
	HookInt		DB 'Hooking interrupt..',13,10,0
        UnHookInt       DB 'Restoring interrupt..',13,10,0
	SetRate		DB 'Setting Samplerate..',13,10,0
	SetDMA		DB 'Setting up DMA Transfer..',13,10,0
	SetSB		DB 'Setting up SB for Transfer..',13,10,0
	PressKey	DB 'Press a key to quit..',13,10,0
        SSpeakers       DB 'Speakers are : ',0
        SOn             DB 'On',13,10,0
        SOff            DB 'Off',13,10,0
        Dot		DB '<SB-INTERRUPT>',13,10,0
	Port		DW 240h
	IRQ		DB 5
	DMA		DB 3
	saveIntSel	DW 0
        saveIntOff	DD 0
        IRQs		DB 02h,03h,05h,07h,0Ah
        DMAs		DB 00h,01h,03h
	IRQtoInt	DB 08h,09h,0Ah,0Bh,0Ch,0Eh,0Fh,70h,71h,72h,73h,74h,75h,76h,77h



;=============================================================
; SBISR
; -----
; SoundBlaster Interrupt Service Routine
; this interrupt is called after each completed dma transfer
; 
;=============================================================
sbisr:	
	cli
	push eax
	push edx
	push edi

	push ds
	push cs
	pop  ds

 	mov  dx,Port
	add  dx,0eh
	in   al,dx		; ACk interrupt to SB by reading port 2xEh

	mov  al,4              	; Do your stuff here, like refilling sound buffers
	mov  edi, OFFSET Dot    ; and restarting playing, etc...
	int  20h 		; Now we just output a message that interrupt has occured.

	mov  al,IRQ
	and  al,0F8h
	jz   sbisr_AckPIC1	; IRQ > 7 ?
	mov  dx,PIC2		; Yes, also ACK interrupt to PIC2
	mov  al,EOI
	out  dx,al
sbisr_AckPIC1:	
	mov  dx,PIC1		; ACK interrupt to PIC1
	mov  al,EOI
	out  dx,al

	pop  ds
	pop  edi
	pop  edx
	pop  eax              
	sti
	iret
	

;=============================================================
; Setup and enable SB interrupt handler
;=============================================================
HookSBint:
	push eax
	push ebx
	push ecx
	push edx
	push edi

	mov  al,4              
	mov  edi, OFFSET HookInt   
	int  20h 

        xor  ax,ax
	mov  al,IRQ
	mov  di,ax        
	mov  ah,[IRQtoINT + di]		; map IRQ to an interrupt
        mov  al,2ah			; save old int
	int  20h
	mov  saveIntSel,bx
	mov  saveIntOff,edi
	
	mov  ah,IRQ
	add  ah,8	
	mov  al,1			; set new int
	mov  bx,cs
	mov  edi,OFFSET sbisr
        int  20h

	mov  al,IRQ			; IRQ > 7?
	and  al,0F8h
	jnz  HookSBInt_OnPIC2		; yes, select PIC2
	mov  dx,PIC1+1			; else select PIC1
	mov  cl,IRQ
	jmp  HookSBInt_EnableInt
HookSBInt_OnPIC2:
	mov  dx,PIC2+1
	mov  cl,IRQ
	sub  cl,8
HookSBInt_EnableInt:
	mov  bl,1			; enable interrupt
	shl  bl,cl
	not  bl
	in   al,dx
	and  al,bl
	out  dx,al

	pop  edi
	pop  edx
	pop  ecx
	pop  ebx
	pop  eax
	ret

;=============================================================
; Restore old interrupt handler
;=============================================================
RestoreSBint:
	push eax
	push ebx
	push ecx
	push edx
	push edi

	mov  al,4              
	mov  edi, OFFSET UnHookInt   
	int  20h 

	xor  ax,ax
	mov  al,IRQ
	mov  di,ax
	mov  ah,[IRQtoINT + di]		; map IRQ to an interrupt
	mov  al,1
	mov  bx,saveIntSel		; restore saved int
	mov  edi,saveIntOff
	int  20h

	mov  al,IRQ			; IRQ > 7?
	and  al,0F8h
	jnz  RestoreSBInt_OnPIC2	; yes, select PIC2
	mov  dx,PIC1+1			; else select PIC1
	mov  cl,IRQ
	jmp  RestoreSBInt_DisableInt
RestoreSBInt_OnPIC2:
	mov  dx,PIC2+1
	mov  cl,IRQ
	sub  cl,8
RestoreSBInt_DisableInt:
	mov  bl,1			; disable interrupt
	shl  bl,cl
	in   al,dx
	or   al,bl
	out  dx,al

	pop  edi
	pop  edx
	pop  ecx
	pop  ebx
	pop  eax
	ret


;=============================================================
; GetSpeakers
;=============================================================
GetSpeakers:
	mov  al,04h
       	mov  edi, OFFSET SSpeakers
       	int  20h   

       	mov  dx,Port
       	call DSPGetSpeaker
       	or   al,al
       	js   GetSpeakers_On

       	mov  al,04h
       	mov  edi, OFFSET SOff
       	int  20h
       	jmp  GetSpeakers_End
GetSpeakers_On:
       	mov  al,04h
       	mov  edi, OFFSET SOn
       	int  20h
GetSpeakers_End:
       	ret



;=============================================================
; GetSettings
;=============================================================
GetSettings:
	push eax
	push edi
	
	call DSPDetectBase	; Detect Base Port
	mov  Port,dx
	
	mov  al,4		; Ask for IRQ
	mov  edi,OFFSET AskIRQ
	int  20h
	mov  al,17h		; wait for key
	int  20h	
	mov  al,1dh		; get key
	int  20h
	sub  al,30h		; ASCII -> Decimal
	xor  ah,ah
	mov  di,ax
	mov  al,[IRQs + di]	; irq to al
	mov  IRQ,al
		
	mov  al,4		; Ask for DMA
	mov  edi,OFFSET AskDMA
	int  20h
	mov  al,17h		; wait for key
	int  20h	
	mov  al,1dh		; get key
	int  20h
	sub  al,30h		; ASCII -> Decimal
	xor  ah,ah
	mov  di,ax
	mov  al,[DMAs + di]	; dma to al
	mov  DMA,al	

	pop  edi
	pop  eax
	ret

;=============================================================
; ShowSettings
;=============================================================
ShowSettings:
	push eax
	push ebx
	push edi

	mov  al,4
	mov  edi,OFFSET SBOnPort
	int  20h
	mov  ax,0205h
	mov  bx,Port
	int  20h

	mov  al,4
	mov  edi,OFFSET SBOnIRQ
	int  20h
	mov  ax,0105h
	mov  bl,IRQ
	int  20h
	
	mov  al,4
	mov  edi,OFFSET SBOnDMA
	int  20h
	mov  ax,0105h
	mov  bl,DMA
	int  20h
	mov  al,4
	mov  edi,OFFSET Return
	int  20h	
	
	pop  edi
	pop  ebx
	pop  eax
	ret

;=============================================================
;=============================================================
;
;  MAIN
;
;=============================================================
;=============================================================

start:
	mov  al, 4               ; Select System32 Service 4 (PrintText)
	mov  edi,OFFSET Welcome  ; Point EDI to our textline
	int  20h                 ; Do System32 Service

	call GetSettings	 ; Get SB settings from user
	
	mov  dx,Port
	call DSPReset		 ; Reset DSP
	jc   Reset_Failed	 ; If failed (no SB?) goto end
	
	call ShowSettings	 ; Show SB settings used
	call DSPSpeakerOn	 ; Turn speakers on 
	call GetSpeakers	 ; and show speaker status

	call HookSBint		 ; Hook SB interrupt
	
	mov  al,4              
	mov  edi, OFFSET SetRate
	int  20h 

        mov  dx,Port		 ; 
	mov  ebx,8000		 ; Set sample rate to 8000Hz
	call DSPSetSampleRate
	
	mov  al,4              
	mov  edi,OFFSET SetDMA
	int  20h 
	
	mov  bx,cs		 ; Get base address of selector cs
	mov  al,2bh
	int  20h
	add  edi,OFFSET Sound	 ; Add offset to sound data
	
        mov  al,dmaModeSINGLE	 ; setup DMA transfer of sound data
        or   al,dmaRdTransfer 	 ; SB reads from memory
	mov  ah,DMA		 ; Use this DMA channel
	mov  cx,4000		 ; size of block
	call DMAInitTransfer	 ; 

	mov  al,4              
	mov  edi, OFFSET SetSB
	int  20h 

	mov  cx,4000
	mov  dx,Port
	mov  al,14h		; setup SB for 8-bit Mono DMA transfer
	call DSPWrite		; write command
	mov  al,cl		; block size lower byte
	call DSPWrite		; 
	mov  al,ch		; block size higher byte
	call DSPWrite		; playing starts immediately after writing this byte
   
	mov  al,4              
	mov  edi, OFFSET PressKey
	int  20h 

	mov  al,17h		; wait for key
	int  20h

	call RestoreSBint
        jmp  SBPlay_End
Reset_Failed:
	mov  al,04h
	mov  edi, OFFSET SBNotFound
	int  20h
SBPlay_End:

	retf
code32  ends

END SBPlay

