/* file_to_v2fs.c */

/*
 * 18/08/1999 c.e. prelz
 * Copyright (C) 1999 V2 Lab Rotterdam
 *
 * Moves a movie from a file to a v2fs partition
 *
 * $Log: file_to_v2fs.c,v $
 * Revision 2.2  1999/12/23 11:15:43  karl
 * GPLized
 *
 * Revision 2.1  1999/09/03 08:49:01  karl
 * Before trying to split the two fields of each frame
 *
 * Revision 1.1  1999/08/18 13:19:03  karl
 * Initial revision
 *
 */

/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <v2gen.h>
#include <v2fs.h>

void usage(void);

int main(int argc,char *argv[])
{
  v2fs *fs;
  unsigned char bfr[256],*pname=NULL,*fname=NULL,*nname=NULL;
  int i,iunit,mov,progr;
  v2fs_movie_head mh;

  while((i=getopt(argc,argv,"p:f:n:"))!=EOF)
  {
    switch(i)
    {
    case 'p':
      pname=safrealloc(pname,strlen(optarg)+1);
      strcpy(pname,optarg);
      break;      
    case 'f':
      fname=safrealloc(fname,strlen(optarg)+1);
      strcpy(fname,optarg);
      break;
    case 'n':
      nname=safrealloc(nname,strlen(optarg)+1);
      strcpy(nname,optarg);
      break;
    default:
      usage();
    }
  }

  if(pname==NULL || fname==NULL)
    usage();

  if((fs=v2fs_open(pname,0))==NULL)
  {
    fprintf(stderr,"%s: error opening v2fs partition <%s>\n",argv[0],pname);
    exit(1);
  }

  if(!strcmp(fname,"-"))
    iunit=0;
  else if((iunit=open(fname,O_RDONLY))<0)
  {
    fprintf(stderr,"%s: could not open input file <%s> (%s)\n",argv[0],
            fname,strerror(errno));
    exit(1);
  }

  if(read(iunit,bfr,strlen(V2SIGN)+1)!=strlen(V2SIGN)+1 ||
     strcmp(bfr,V2SIGN))
  {
    fprintf(stderr,"%s: input file has wrong signature!\n",argv[0]);
    exit(1);
  }
  if(read(iunit,bfr,strlen(V2FSVERSION)+1)!=strlen(V2FSVERSION)+1 ||
     strcmp(bfr,V2FSVERSION))
  {
    bfr[strlen(V2FSVERSION)]='\0';
    fprintf(stderr,"%s: input file has wrong version! (%s)\n",argv[0],bfr);
    exit(1);
  }
  
  if(read(iunit,bfr,128)!=128)
  {
    fprintf(stderr,"%s: could not read mhead structure!\n",argv[0]);
    exit(1);
  }
  mh=*((v2fs_movie_head *)bfr);

/*
 * Turn off deletion...
 */

	mh.g.flags=0;

/*
 * Substitute new name if needed
 */
  
  if(nname!=NULL)
  {
    if(strlen(nname)>=V2FS_DESCLEN)
    {
      fprintf(stderr,"%s: proposed name <%s> is too long (max. %d chars)!\n",
              argv[0],nname,V2FS_DESCLEN);
      exit(1);
    }
    strcpy(mh.g.filename,nname);
  }

  mov=v2fs_whichmovie(fs,mh.g.filename,NULL);
  if(mov>=0)
  {
    fprintf(stderr,"%s: Movie <%s> already exists in %s\n",argv[0],mh.g.filename,
            pname);
    exit(1);
  }

/*
 * Ask the fs driver for a suitable progr. # and start position
 */

  if((progr=v2fs_findspace(fs,mh.g.n_sects,&mh.g.data_start))<0)
  {
    fprintf(stderr,"%s: No space on <%s>!\n",argv[0],pname);
    exit(1);
  }

/*
 * then simply dump the movie
 */

  fprintf(stderr,"Saving <%s> as <%s>.\n",fname,mh.g.filename);
  v2fs_locateblk(fs->unit,mh.g.data_start);
  for(i=0;i<mh.g.n_sects;i++)
  {
    if(read(iunit,v2fs_blk,V2BLKSIZE)!=V2BLKSIZE)
    {
      fprintf(stderr,"%s: input file too short! (reading sector #%d)\n",
              argv[0],i+1);
      exit(1);
    }
    v2fs_writenext(fs->unit);
  }
  close(iunit);

/*
 * and at last the mhead
 */

  fprintf(stderr,"Updating mhead structure.\n");
  if(v2fs_update_mhead(fs,&mh,progr))
  {
    fprintf(stderr,"Error updating mhead!\n");
    exit(1);
  }

  fprintf(stderr,"Done!\n");
	return 0;
}

void usage(void)
{
  fprintf(stderr,"Usage: file_to_v2fs -p <v2fs partition> -f <input-file> [-n <new name>]\n");
  exit(1);
}

  
