/* file_to_v2obj.c */

/*
 * 7/9/1999 c.e. prelz
 * Copyright (C) 1999 V2 Lab Rotterdam
 *
 * This program creates a v2fs-type object from a given file and type.
 *
 * $Log: file_to_v2obj.c,v $
 * Revision 1.2  1999/12/23 11:15:43  karl
 * GPLized
 *
 * Revision 1.1  1999/09/09 11:40:17  karl
 * Initial revision
 *
 */

/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <time.h>
#include <v2gen.h>
#include <v2fs.h>

void usage(void);

int main(int argc,char *argv[])
{
  int i,iunit,ounit,fsize;
  unsigned char *fname=NULL,*oname=NULL,*nname=NULL,*type=NULL;
  v2fs_movie_head mhead;
  
  while((i=getopt(argc,argv,"f:t:o:n:"))!=EOF)
  {
    switch(i)
    {
    case 'f':
      fname=safrealloc(fname,strlen(optarg)+1);
      strcpy(fname,optarg);
      break;      
    case 't':
      type=safrealloc(type,strlen(optarg)+1);
      strcpy(type,optarg);
      break;
    case 'o':
      oname=safrealloc(oname,strlen(optarg)+1);
      strcpy(oname,optarg);
      break;      
    case 'n':
      nname=safrealloc(nname,strlen(optarg)+1);
      strcpy(nname,optarg);
      break;      
    default:
      usage();
    }
  }
      
  if(fname==NULL || type==NULL || oname==NULL)
    usage();

  if(nname==NULL)
  {
    if(strlen(fname)>=V2FS_DESCLEN)
    {
      fprintf(stderr,"%s: The name of the input file (%s) is too long to be used as movie name. Use option -n.\n",
              argv[0],fname);
      exit(1);
    }
  }
  else
  {
    if(strlen(nname)>=V2FS_DESCLEN)
    {
      fprintf(stderr,"%s: The proposed name (%s) is too long to be used as movie name.\n",
              argv[0],nname);
      exit(1);
    }
  }

  if((iunit=open(fname,O_RDONLY))<0)
  {
    fprintf(stderr,"%s: could not open input file <%s> (%s)\n",argv[0],
            fname,strerror(errno));
    exit(1);
  }
  fsize=lseek(iunit,0L,SEEK_END);
  lseek(iunit,0L,SEEK_SET);
  
  if(!strcmp(oname,"-"))
    ounit=1;
  else if((ounit=open(oname,O_WRONLY|O_TRUNC|O_CREAT,0644))<0)
  {
    fprintf(stderr,"%s: could not open output file <%s> (%s)\n",argv[0],
            oname,strerror(errno));
    exit(1);
  }

  write(ounit,V2SIGN,strlen(V2SIGN)+1);
  write(ounit,V2FSVERSION,strlen(V2FSVERSION)+1);

  bzero(&mhead,sizeof(mhead));

  memcpy(mhead.g.type,type,4);
  mhead.g.n_sects=((fsize-1)/V2BLKSIZE)+1;
  strcpy(mhead.g.filename,(nname!=NULL) ? nname : fname);
  mhead.u.c.last_sect_size=fsize%V2BLKSIZE;

  mhead.g.time_cr=time(NULL);
  
  memcpy(v2fs_blk,&mhead,sizeof(mhead));
  write(ounit,v2fs_blk,128);

/*
 * Then the data stuff. In blocks of 512 - same as movie
 */

  for(i=0;i<mhead.g.n_sects;i++)
  {
    if(read(iunit,v2fs_blk,V2BLKSIZE)<=0)
    {
      fprintf(stderr,"%s: error writing data sector #%d (%s)\n",
              argv[0],i+1,strerror(errno));
      exit(1);
    }
    write(ounit,v2fs_blk,V2BLKSIZE);
  }
  close(iunit);
  close(ounit);

  return 0;
}

void usage(void)
{
  fprintf(stderr,"Usage: file_to_v2obj -f <file name> -t <type (4 chars)> -o <output file> [-n <new name>]\n");
  exit(1);
}
  
