/* part_info.c */

/*
 * 10/08/1999 c.e. prelz
 * Copyright 1999 V2 Labs
 *
 * Prints info on the movies included in a partition
 *
 * $Log: part_info.c,v $
 * Revision 2.2  1999/12/23 11:15:43  karl
 * GPLized
 *
 * Revision 2.1  1999/09/03 08:49:01  karl
 * Before trying to split the two fields of each frame
 *
 * Revision 1.2  1999/08/25 14:11:59  karl
 * Before adapting to std lib call
 *
 * Revision 1.1  1999/08/10 11:53:42  karl
 * Initial revision
 *
 */

/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <v2gen.h>
#include <v2fs.h>

int main(int argc,char *argv[])
{
  v2fs *fs;
  v2fs_movie_head *mhead;
  int i;
  unsigned char *ptr;
  
  if(argc<2)
  {
    fprintf(stderr,"Usage: %s <v2fs partition>\n",argv[0]);
    exit(1);
  }

  if((fs=v2fs_open(argv[1],1))==NULL)
  {
    fprintf(stderr,"%s: error opening v2fs partition <%s>\n",argv[0],argv[1]);
    exit(1);
  }

  printf("Partition <%s>\n",argv[1]);
  printf("Max. objects: %ld. Currently used: %ld (%.1f%%)\n",fs->head.max_objects,
         fs->head.cur_objects,fs->head.cur_objects*100.0/fs->head.max_objects);
  printf("Max. sectors: %ld. Currently used: %ld (%.1f%%)\n",fs->head.max_sectors,
         fs->head.first_avail_sector,
         fs->head.first_avail_sector*100.0/fs->head.max_sectors);

  for(i=0;i<fs->head.cur_objects;i++)
  {
    mhead=&(fs->mheads[i]);

    if(mhead->g.flags&=V2FS_FLGG_DELETED)
    {
      printf("#%d: ***DELETED***\n",i+1);
      continue;
    }

    if(!memcmp(mhead->g.type,V2FS_MBUZ_TYPE,4))
    {
      ptr=ctime(&mhead->g.time_cr);
      ptr[strlen(ptr)-1]='\0';
      
      printf("#%d: %s. %ld frames. %ld sectors, starting at %ld. Created %s.\n",
             i+1,mhead->g.filename,mhead->u.m.n_frames,mhead->g.n_sects,
             mhead->g.data_start,ptr);
      printf("  %d fps, img %dX%d\n",mhead->u.m.fps,mhead->u.m.x_size,mhead->u.m.y_size);
      printf("  Audio: %hd samples/s, %hd bytes/f, %s, %d bits\n",
             mhead->u.m.audio_samp_per_sec,mhead->u.m.audio_byte_per_frame,
             mhead->u.m.flags&V2FS_FLGM_STEREO ? "Stereo" : "Mono",
             mhead->u.m.flags&V2FS_FLGM_AUDIO_16BITS ? 16 : 8);
    }
    else
      printf("#%d: %s. Type <%.4s>. %ld sectors.\n",i+1,mhead->g.filename,mhead->g.type,
             mhead->g.n_sects);
  }

  return 0;
}

    

  
  
