/* v2fs.c */

/*
 * 05/08/1999 c.e. prelz
 * Copyright 1999 V2 Labs
 *
 * Implementation of the filesystem to hold v2 movies
 *
 * $Log: v2fs.c,v $
 * Revision 2.2  1999/12/23 11:15:43  karl
 * GPLized
 *
 * Revision 2.1  1999/09/03 08:49:01  karl
 * Before trying to split the two fields of each frame
 *
 * Revision 1.5  1999/08/27 08:36:39  karl
 * Before at last seeing how to deal with deleted items
 *
 * Revision 1.4  1999/08/16 13:37:12  karl
 * Before splitting head struct in two
 *
 * Revision 1.3  1999/08/13 10:19:40  karl
 * Before adding sub to get positioned to given frame
 *
 * Revision 1.2  1999/08/10 08:06:31  karl
 * Before adding control on /proc file for presence of partition
 *
 * Revision 1.1  1999/08/06 10:25:33  karl
 * Initial revision
 *
 */

/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/sysmacros.h>
#include <sys/file.h>
#include <linux/major.h>
#include <linux/unistd.h>
#include <v2gen.h>
#include <v2fs.h>

unsigned char v2fs_blk[V2BLKSIZE];

static int openfile(unsigned char *path,unsigned long *fpart,unsigned char readonly);

/*extern FILE *zlo;*/

int v2fs_init(unsigned char *path,unsigned long max_objects,
              unsigned char sect16,unsigned char sect32)
{
  int unit,i;
  v2fs_head_head *hd;
  unsigned char *ptr;
  unsigned long part_size;
    
  if((unit=openfile(path,&part_size,0))<0)
  {
    fprintf(stderr,"v2fs_init: Could not open (%s)\n",path);
    return -1;
  }

/*
 * If the first sector is not all zeroes, refuse to init
 */

  if(v2fs_readblk(unit,0))
  {
    fprintf(stderr,"v2fs_init: Could not read 1st block (%s)\n",path);
    close(unit);
    return -1;
  }

  for(i=0;i<V2BLKSIZE;i++)
    if(v2fs_blk[i])
      break;
  if(i<V2BLKSIZE)
  {
    fprintf(stderr,"v2fs_init: 1st sector not zeroed! (%s)\n",path);
    close(unit);
    return -1;
  }

/*
 * Ok we all agree. Let's do the massacre
 */
  
  memset(v2fs_blk,0,V2BLKSIZE);

  ptr=v2fs_blk+V2SIGNSKIP;
  
  strcpy(ptr,V2SIGN);
  ptr+=strlen(ptr)+1;
  
  strcpy(ptr,V2FSVERSION);
  ptr+=strlen(ptr)+1;

  hd=(v2fs_head_head *)ptr;
  
  hd->max_objects=max_objects;
  hd->cur_objects=0L;

  hd->sect16=sect16;
  hd->sect32=sect32;
  hd->first_data_sector=hd->sect16+hd->sect32+1;
  
/*
 * See how many sectors
 */

  hd->max_sectors=(unsigned long)((loff_t)part_size*1024L/V2BLKSIZE);
  hd->first_avail_sector=hd->first_data_sector+((max_objects-1)>>2)+1;    
    
  i=v2fs_writeblk(unit,0);

  close(unit);
  
  return i;
}

extern v2fs *v2fs_open(unsigned char *path,unsigned char read_only)
{
  v2fs *fshead;
  int i,j,k;
  unsigned char *ptr;
  unsigned long fsize;

  if((i=openfile(path,&fsize,read_only))<0)
  {
    fprintf(stderr,"v2fs_open: Could not open (%s) for reading%s\n",path,
            read_only ? "" : "/writing");
    return NULL;
  }

/*
 * Check signature
 */

  if(v2fs_readblk(i,0))
  {
    fprintf(stderr,"v2fs_open: Could not read 1st block (%s)\n",path);
    close(i);
    return NULL;
  }

  ptr=v2fs_blk+V2SIGNSKIP;
  
  if(strcmp(ptr,V2SIGN))
  {
    fprintf(stderr,"v2fs_open: Signature wrong on %s (begins with %.2x %.2x %.2x %.2x)\n",
            path,ptr[0],ptr[1],ptr[2],ptr[3]);
    close(i);
    return NULL;
  }
  ptr+=strlen(V2SIGN)+1;
  if(strcmp(ptr,V2FSVERSION))
  {
    fprintf(stderr,"v2fs_open: version wrong on %s (%s)\n",path,ptr);
    close(i);
    return NULL;
  }
  ptr+=strlen(V2FSVERSION)+1;
  
  fshead=safalloc(sizeof(v2fs));
  fshead->unit=i;

  memcpy(&fshead->head,ptr,sizeof(v2fs_head_head));

/*
 * Load the current existing movie heads in memory
 */

  if(fshead->head.cur_objects)
  {
    fshead->mheads=safalloc(sizeof(v2fs_movie_head)*fshead->head.cur_objects);
    for(i=0,j=fshead->head.first_data_sector;i<fshead->head.cur_objects;i+=4,j++)
    {
      if(v2fs_readblk(fshead->unit,j)<0)
      {
        fprintf(stderr,"v2fs_open: Colud not read sector #%d (%s)\n",j,path);
        close(fshead->unit);
        safree(fshead);
        return NULL;
      }
      for(k=0;k<4;k++)
      {
        if(i+k>=fshead->head.cur_objects)
          break;
        memcpy(&(fshead->mheads[i+k]),v2fs_blk+128*k,sizeof(v2fs_movie_head));
      }
    }
  }
  
  return fshead;
}

int v2fs_close(v2fs *fs)
{
  if(flock(fs->unit,LOCK_UN)<0)
  {
    perror("v2fs_close - unlocking unit");
    return -1;
  }
  if(close(fs->unit)<0)
  {
    perror("v2fs_close - closing unit");
    return -1;
  }
  safree(fs->mheads);
  safree(fs);
  
  return 0;
}

int v2fs_locateblk(int unit,loff_t sect)
{
  sect<<=V2BLKBITS; /* obtain bytes */
  
  if(llseek(unit,sect,SEEK_SET)==(loff_t)-1)
  {
    perror("llseek");
    return -1;
  }
  return 0;
}
  
int v2fs_readblk(int unit,loff_t sect)
{
  if(v2fs_locateblk(unit,sect))
    return -1;

  return v2fs_readnext(unit);
}

inline int v2fs_readnext(int unit)
{
  if(read(unit,v2fs_blk,V2BLKSIZE)!=V2BLKSIZE)
    return -2;

  return 0;
}

int v2fs_writeblk(int unit,loff_t sect)
{
  if(v2fs_locateblk(unit,sect))
    return -1;

  return v2fs_writenext(unit);
}

inline int v2fs_writenext(int unit)
{
  if(write(unit,v2fs_blk,V2BLKSIZE)!=V2BLKSIZE)
  {
    perror("v2fs_blk");
    return -1;
  }
  
  return 0;
}

inline int v2fs_skipsectors(int unit,unsigned long sects)
{
  return llseek(unit,(loff_t)sects*V2BLKSIZE,SEEK_CUR);
}

v2fs_movie_head *v2fs_gethead(v2fs *fs,int progr)
{
  v2fs_movie_head *movh;
  
  if(progr<0 || progr>=fs->head.cur_objects)
    return NULL;

  if(v2fs_readblk(fs->unit,fs->head.first_data_sector+progr/4))
    return NULL;

  movh=safalloc(sizeof(v2fs_movie_head));
  memcpy(movh,v2fs_blk+128*(progr%4),sizeof(v2fs_movie_head));

  return movh;
}

signed long v2fs_findspace(v2fs *fs,unsigned long n_sects,unsigned long *start_sector)
{
  int i,j;
  signed long mindiff=100000000L;

  for(i=0,j=-1;i<fs->head.cur_objects;i++)
    if(fs->mheads[i].g.flags&V2FS_FLGG_DELETED &&
       fs->mheads[i].g.n_sects>=n_sects &&
       (fs->mheads[i].g.n_sects-n_sects)<mindiff)
    {
      j=i;
      mindiff=fs->mheads[i].g.n_sects-n_sects;
    }
  if(j>=0)
  {
    *start_sector=fs->mheads[j].g.data_start;
    return j;
  }

  if(fs->head.cur_objects>=fs->head.max_objects)
  {
    fprintf(stderr,"v2fs_findspace; sorry... too many movies on disk!\n");
    return -1;
  }
  if(fs->head.max_sectors-fs->head.first_avail_sector<n_sects)
  {
    fprintf(stderr,"v2fs_findspace; sorry... no more space on disk! (%ld/%ld/%ld)\n",
            fs->head.max_sectors,fs->head.first_avail_sector,n_sects);
    return -1;
  }

  *start_sector=fs->head.first_avail_sector;
  return fs->head.cur_objects;
}

unsigned long v2fs_cursector(int unit)
{
  loff_t cur=llseek(unit,(loff_t)0,SEEK_CUR);

  return (unsigned long)(cur>>V2BLKBITS);
}

void v2fs_update_hhead(v2fs *fs,v2fs_head_head *hhead)
{
  v2fs_head_head *hptr;
  
  v2fs_readblk(fs->unit,0);
  hptr=(v2fs_head_head *)(v2fs_blk+V2SIGNSKIP+strlen(V2SIGN)+strlen(V2FSVERSION)+2);
  *hptr=*hhead;
  v2fs_writeblk(fs->unit,0);
}

int v2fs_update_mhead(v2fs *fs,v2fs_movie_head *mhead,signed long progr)
{
  if(progr<0 || progr>fs->head.cur_objects || progr>=fs->head.max_objects)
  {
    fprintf(stderr,"v2fs_update_mhead: Progr %ld can't be accepted!\n",progr);
    return -1;
  }
  if(!fs->head.cur_objects || progr>=fs->head.cur_objects-1)
  {
    fs->head.cur_objects=progr+1; 
    fs->head.first_avail_sector=mhead->g.data_start+mhead->g.n_sects;

    v2fs_update_hhead(fs,&fs->head);
  }

  v2fs_readblk(fs->unit,fs->head.first_data_sector+(progr>>2));  
  *(v2fs_movie_head *)(v2fs_blk+128*(progr%4))=*mhead;
  v2fs_writeblk(fs->unit,fs->head.first_data_sector+(progr>>2));

  return 0;
}

unsigned long v2fs_position(v2fs *fs,unsigned long mov,unsigned long frame)
{
/*fprintf(zlo,"() Asking for %d/%d\n",mov,frame);fflush(zlo);*/
 
  if(mov<0 || mov>=fs->head.cur_objects)
    return-1;    
  if(frame<0 || frame>=fs->mheads[mov].u.m.n_frames)
    return-1;
  v2fs_readblk(fs->unit,fs->mheads[mov].g.data_start+frame/128);

  return fs->mheads[mov].g.data_start+
    *(unsigned long *)(v2fs_blk+sizeof(unsigned long)*(frame%128));
}

int v2fs_validitems(v2fs *fs,unsigned char *type)
{
  int to_ret,i;

  for(i=0,to_ret=0;i<fs->head.cur_objects;i++)
    if(!(fs->mheads[i].g.flags&V2FS_FLGG_DELETED) &&
       (type==NULL || !memcmp(fs->mheads[i].g.type,type,4)))
      to_ret++;

  return to_ret;
}

int v2fs_whichmovie(v2fs *fs,unsigned char *name,unsigned char *type)
{
  int i;
  
  for(i=0;i<fs->head.cur_objects;i++)
    if(!(fs->mheads[i].g.flags&V2FS_FLGG_DELETED) &&
       (type==NULL || !memcmp(fs->mheads[i].g.type,type,4)) &&
       !strcmp(name,fs->mheads[i].g.filename))
      return i;

  return -1;
}

/*
 * If mhead is non-null, do not close unit and return it in runit and the mhead
 * in mhead
 */

unsigned char *v2fs_check_file(unsigned char *path,unsigned long *tstamp,
                               v2fs_movie_head *mhead,int *runit)
{
  int unit;
  unsigned char bfr[256];
  v2fs_movie_head *mh;
  static unsigned char *to_ret=NULL;

  if((unit=open(path,O_RDONLY))<0)
  {
    perror(path);
    return NULL;
  }
  if(read(unit,bfr,strlen(V2SIGN)+1)!=strlen(V2SIGN)+1 ||
     strcmp(bfr,V2SIGN))
  {
    fprintf(stderr,"v2fs_check_file: file <%s> has wrong signature!\n",path);
    close(unit);
    return NULL;
  }
  if(read(unit,bfr,strlen(V2FSVERSION)+1)!=strlen(V2FSVERSION)+1 ||
     strcmp(bfr,V2FSVERSION))
  {
    bfr[strlen(V2FSVERSION)]='\0';
    fprintf(stderr,"v2fs_check_file: file <%s> has wrong version! (%s)\n",path,bfr);
    close(unit);
    return NULL;
  }
  if(read(unit,bfr,128)!=128)
  {
    fprintf(stderr,"v2fs_check_file: could not read mhead structure!\n");
    close(unit);
    return NULL;
  }

  mh=(v2fs_movie_head *)bfr;
  if(mhead!=NULL)
  {
    *runit=unit;
    *mhead=*mh;
  }
  else
    close(unit);

  *tstamp=mh->g.time_cr;
  
  to_ret=safrealloc(to_ret,strlen(mh->g.filename)+1);
  strcpy(to_ret,mh->g.filename);
  return to_ret;
}

int v2fs_delete(v2fs *fs,unsigned char *name)
{
  int pos;

  if((pos=v2fs_whichmovie(fs,name,NULL))<0)
  {
    fprintf(stderr,"v2fs_delete: movie <%s> is not in the filesystem!\n",name);
    return -1;
  }
  
/*
 * Ok let's do it
 * Special case when it's the last one!!
 */

  if(pos==fs->head.cur_objects-1)
  {
    fs->head.first_avail_sector=fs->mheads[pos].g.data_start;
    fs->head.cur_objects--;
    v2fs_update_hhead(fs,&fs->head);
  }
  else
  {
    fs->mheads[pos].g.flags|=V2FS_FLGG_DELETED;
    
    fs->mheads[pos].g.n_sects=
      fs->mheads[pos+1].g.data_start-fs->mheads[pos].g.data_start; /* set max size
                                                                      again */
    v2fs_update_mhead(fs,&fs->mheads[pos],pos);
  }

  return 0;
}

static int openfile(unsigned char *path,unsigned long *psize,unsigned char readonly)
{
  struct stat st;
  int i,j,found;
  FILE *fpart;
  unsigned char bfr[256];
  
  if((stat(path,&st))<0)
  {
    perror(path);
    return -1;
  }

  if(!S_ISBLK(st.st_mode))
  {
    fprintf(stderr,"v2fs_open: Not a block device (%s)\n",path);
    return -1;
  }

  if(access(path,(readonly) ? R_OK : R_OK|W_OK))
  {
    fprintf(stderr,"v2fs_open: Can't access %s\n",path);
    return -1;
  }
  
/*
 * Device must start with /dev/!
 */

  if(strncmp(path,"/dev/",5))
  {
    fprintf(stderr,"v2fs_open: %s must be in the /dev/ directory!\n",path);
    return -1;
  }

/*
 * Is it a diskette?
 */

	if(!strcmp(path+5,"fd0") || !strcmp(path+5,"fd1"))
		found=1;
	else
	{
		
/*
 * Open /proc/partitions, and find if our name is there
 */

		if((fpart=fopen("/proc/partitions","r"))==NULL)
		{
			perror("/proc/partitions");
			return -1;
		}
		fgets(bfr,256,fpart);
		fgets(bfr,256,fpart); /* skip heading */
		found=0;
		while(fscanf(fpart,"%d %d %ld %s\n",&i,&j,psize,bfr)==4)
			if(!strcmp(bfr,path+5))
			{
				found=1;
				break;
			}
		fclose(fpart);
	}
	
  if(!found)
  {
    fprintf(stderr,"v2fs_open: Not a disk partition! %s\n",path);
    return -1;
  }

  if((i=open(path,(readonly) ? O_RDONLY : O_RDWR))<0)
  {
    perror(path);
    return -1;
  }
  if(flock(i,((readonly) ? LOCK_SH : LOCK_EX)|LOCK_NB))
  {
    fprintf(stderr,"v2fs_open: %s is locked!\n",path);
    close(i);
    return -1;
  }

  return i;
}

