/* v2fs_defrag.c */

/*
 * 23/09/1999 c.e. prelz
 * Copyright (C) 1999 V2_Lab Rotterdam
 *
 * Defragments a v2fs partition. All existing objects are moved to the beginning
 * of the partition.
 *
 * $Log: v2fs_defrag.c,v $
 * Revision 1.2  1999/12/23 11:15:43  karl
 * GPLized
 *
 * Revision 1.1  1999/12/23 11:05:15  karl
 * Initial revision
 *
 */

/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <v2gen.h>
#include <v2fs.h>

void usage(void);

int main(int argc,char *argv[])
{
  v2fs *fs;
  unsigned char *pname=NULL;
  int orig,reloc,forig,fdest,i,j;

  while((i=getopt(argc,argv,"p:"))!=EOF)
  {
    switch(i)
    {
    case 'p':
      pname=safrealloc(pname,strlen(optarg)+1);
      strcpy(pname,optarg);
      break;
    default:
      usage();
    }
  }

  if(pname==NULL)
    usage();

  if((fs=v2fs_open(pname,0))==NULL)
  {
    fprintf(stderr,"%s: error opening v2fs partition <%s>\n",argv[0],pname);
    exit(1);
  }

  fs->head.first_avail_sector=fs->head.first_data_sector+
    ((fs->head.max_objects-1)>>2)+1;
  
  for(orig=0,reloc=0;orig<fs->head.cur_objects;orig++)
  {
    if(fs->mheads[orig].g.flags&V2FS_FLGG_DELETED)
      continue;

    printf("Relocating #%d - <%s>\n",orig+1,fs->mheads[orig].g.filename);
    
    for(i=0,forig=fs->mheads[orig].g.data_start,fdest=fs->head.first_avail_sector;
        i<fs->mheads[orig].g.n_sects;
        i++,forig++,fdest++)
    {
      v2fs_readblk(fs->unit,forig);
      v2fs_writeblk(fs->unit,fdest);
    }
    
    fs->mheads[orig].g.data_start=fs->head.first_avail_sector;
    fs->head.first_avail_sector+=fs->mheads[orig].g.n_sects;

    fs->mheads[reloc]=fs->mheads[orig];

    reloc++;
  }

  fs->head.cur_objects=reloc;

/*
 * Now save the various heads
 */

  v2fs_locateblk(fs->unit,fs->head.first_data_sector);
  
  for(i=0;i<fs->head.cur_objects;i+=4)
  {
    for(j=0;j<4;j++)
      memcpy(v2fs_blk+128*j,&fs->mheads[i+j],sizeof(v2fs_movie_head));
    v2fs_writenext(fs->unit);
  }

  v2fs_update_hhead(fs,&fs->head);

  v2fs_close(fs);

  return 0;
}
      
void usage(void)
{
  fprintf(stderr,"Usage: v2fs_defrag -p <v2fs partition>\n");
  exit(1);
}
