/* v2fs_rename.c */

/*
 * 21/08/1999 c.e. prelz
 * Copyright (C) 1999 V2 Lab Rotterdam
 *
 * Changes the name of a v2fs movie
 *
 * $Log: v2fs_rename.c,v $
 * Revision 2.3  1999/12/23 11:15:43  karl
 * GPLized
 *
 * Revision 2.2  1999/09/09 08:56:38  karl
 * Now can also change type
 *
 * Revision 2.1  1999/09/03 08:49:01  karl
 * Before trying to split the two fields of each frame
 *
 * Revision 1.1  1999/08/27 08:52:17  karl
 * Initial revision
 *
 */

/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <v2gen.h>
#include <v2fs.h>

void usage(void);

void main(int argc,char *argv[])
{
  v2fs *fs;
  unsigned char *pname=NULL,*oname=NULL,*nname=NULL,*type=NULL;
  int i,mov;

  while((i=getopt(argc,argv,"p:o:n:t:"))!=EOF)
  {
    switch(i)
    {
    case 'p':
      pname=safrealloc(pname,strlen(optarg)+1);
      strcpy(pname,optarg);
      break;      
    case 'o':
      oname=safrealloc(oname,strlen(optarg)+1);
      strcpy(oname,optarg);
      break;      
    case 'n':
      nname=safrealloc(nname,strlen(optarg)+1);
      strcpy(nname,optarg);
      break;      
    case 't':
      type=safrealloc(type,strlen(optarg)+1);
      strcpy(type,optarg);
      break;      
    default:
      usage();
    }
  }
      
  if(pname==NULL || oname==NULL || (nname==NULL && type==NULL))
    usage();
  if(type!=NULL && strlen(type)!=4)
  {
    fprintf(stderr,"Type must be 4 characters long!\n");
    usage();
  }

  if((fs=v2fs_open(pname,0))==NULL)
  {
    fprintf(stderr,"%s: error opening v2fs partition <%s>\n",argv[0],pname);
    exit(1);
  }

/*
 * Find where the movie is
 */

  mov=v2fs_whichmovie(fs,oname,NULL);

  if(mov<0)
  {
    fprintf(stderr,"%s: could not find movie <%s>\n",argv[0],oname);
    exit(1);
  }

  if(nname!=NULL)
  {
    if(strlen(nname)>=V2FS_DESCLEN)
    {
      fprintf(stderr,"%s: Proposed new name (%s) is too long! Max. %d chars\n",
              argv[0],nname,V2FS_DESCLEN-1);
      exit(1);
    }
    
/*
 * See if new name is already used
 */

    if(v2fs_whichmovie(fs,nname,NULL)>=0)
    {
      fprintf(stderr,"%s: A movie with name <%s> already exists!\n",argv[0],nname);
      exit(1);
    }

    strcpy(fs->mheads[mov].g.filename,nname);
  }

  if(type!=NULL)
    memcpy(fs->mheads[mov].g.type,type,4);
    
  if(v2fs_update_mhead(fs,&fs->mheads[mov],mov))
    fprintf(stderr,"%s: error in update_mhead!\n",argv[0]);
  else
    fprintf(stderr,"Change was performed successfully.\n");
}
  
void usage(void)
{
  fprintf(stderr,"Usage: v2fs_rename -p <v2fs partition> -o <old name> [-n <new name> | -t <new type]\n");
  exit(1);
}
