/* v2fs_to_file.c */

/*
 * 18/08/1999 c.e. prelz
 * Copyright (C) 1999 V2 Lab Rotterdam
 *
 * Moves a movie from a v2fs partition to a file
 *
 * $Log: v2fs_to_file.c,v $
 * Revision 2.2  1999/12/23 11:15:43  karl
 * GPLized
 *
 * Revision 2.1  1999/09/03 08:49:01  karl
 * Before trying to split the two fields of each frame
 *
 * Revision 1.1  1999/08/18 12:25:00  karl
 * Initial revision
 *
 */

/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <v2gen.h>
#include <v2fs.h>

void usage(void);

void main(int argc,char *argv[])
{
  v2fs *fs;
  int mov,i,ounit;
  unsigned char *pname=NULL,*mname=NULL,*fname=NULL,mhead_bfr[128];

  while((i=getopt(argc,argv,"p:m:f:"))!=EOF)
  {
    switch(i)
    {
    case 'p':
      pname=safrealloc(pname,strlen(optarg)+1);
      strcpy(pname,optarg);
      break;      
    case 'm':
      mname=safrealloc(mname,strlen(optarg)+1);
      strcpy(mname,optarg);
      break;
    case 'f':
      fname=safrealloc(fname,strlen(optarg)+1);
      strcpy(fname,optarg);
      break;
    default:
      usage();
    }
  }
        
  if(pname==NULL || mname==NULL || fname==NULL)
    usage();

  if((fs=v2fs_open(pname,1))==NULL)
  {
    fprintf(stderr,"%s: error opening v2fs partition <%s>\n",argv[0],pname);
    exit(1);
  }
    
  mov=v2fs_whichmovie(fs,mname,NULL);
  if(mov<0)
  {
    fprintf(stderr,"%s: could not find movie <%s>\n",argv[0],mname);
    exit(1);
  }

  if(!strcmp(fname,"-"))
    ounit=1;
  else if((ounit=open(fname,O_WRONLY|O_TRUNC|O_CREAT,0644))<0)
  {
    fprintf(stderr,"%s: could not open output file <%s> (%s)\n",argv[0],
            fname,strerror(errno));
    exit(1);
  }

  write(ounit,V2SIGN,strlen(V2SIGN)+1);
  write(ounit,V2FSVERSION,strlen(V2FSVERSION)+1);

  bzero(mhead_bfr,128);
  memcpy(mhead_bfr,&(fs->mheads[mov]),sizeof(v2fs_movie_head));
  write(ounit,mhead_bfr,128);

  v2fs_locateblk(fs->unit,fs->mheads[mov].g.data_start);
  for(i=0;i<fs->mheads[mov].g.n_sects;i++)
  {
    v2fs_readnext(fs->unit);
    write(ounit,v2fs_blk,V2BLKSIZE);
  }

  close(ounit);
}

void usage(void)
{
  fprintf(stderr,"Usage: v2fs_to_file -p <v2fs partition> -m <movie name> -f <output-file>\n");
  exit(1);
}


