;================================================
; my constants for v2os coding
; ASCII_table/c00l rebzzz
; last updated: 18 dec 1999 17:37
;================================================

;------------------------------------------------
; System32 Services
; ~~~~~~~~~~~~~~~~~
;------------------------------------------------
S32_NULLF       equ     00d     ; Null Function
S32_SETINT      equ     01d     ; Set Interrupt Vector
S32_RESERVED    equ     02d     ; Reserved
S32_RMINT       equ     03d     ; Execute a RealMode Interrupt
S32_TEXTOUT     equ     04d     ; Send text to the standard outputdevice
S32_HEXOUT      equ     05d     ; Send hex-number to standard outputdevice
S32_READSEC     equ     06d     ; Read physical disk sector(s)
S32_WRITESEC    equ     07d     ; Write physical disk sector(s)
S32_GETSELECTOR equ     08d     ; Request Selector
S32_GETPARTINFO equ     09d     ; Get partition information
S32_GETFILEINFO equ     10d     ; Get FileInformation
S32_CREATEFILE  equ     11d     ; Create New File
S32_SPLITFNAME  equ     12d     ; File part/name Splitter
S32_QBOOT       equ     13d     ; QBoot
S32_ALLOC       equ     14d     ; Allocate Memory+selector
S32_DELSELECTOR equ     15d     ; Delete Selector
S32_MODINFO     equ     16d     ; Get Module Info
S32_ANNOUNCE    equ     17d     ; Announce Server
S32_DELSERV     equ     18d     ; Remove Server
S32_GETSERVCNT  equ     19d     ; Query Amount of Servers
S32_GETSERVMOD  equ     20d     ; Get Server-Module
S32_MODSERVICE  equ     21d     ; Call ModuleServices
S32_RMINT2      equ     22d     ; RM Int (Without Register Copy)
S32_WAITKEY     equ     23d     ; Wait for Keypress
S32_LOOK        equ     24d     ; Set V2_OS 'Look'
S32_SHELLCMD    equ     25d     ; Shell Command
S32_DELAY       equ     26d     ; Delay
S32_ANNOUNCEDISK        equ     27d     ; Announce Disk Interface
S32_DUMPDISKBUFFER      equ     28d     ; Diskbuffer Dump
S32_GETKEY      equ     29d     ; Get Key from Keyboard Buffer
S32_STRCMP      equ     30d     ; Compare String
S32_UCASE       equ     31d     ; UCase String
S32_GETPCIDEVCNT        equ     32d     ; Get Amount of Devices in
                                ; PCI(Sub)Class
S32_GETPCICARDRES       equ     33d     ; Get Resources used by a PCI Card
S32_GETPCICARDIO        equ     34d     ; Get I/O Resources used by a PCI Card
S32_GETPCICARDMEM       equ     35d     ; Get Memory Resources used by a PCI
                                ; Card
S32_GETIRQMASK 		equ	36d	; Get IRQ Mask
S32_SETIRQMASK 		equ	 37d	; Set IRQ Mask
S32_ALTTEXTOUT 		equ	 38d	; Write text to alternate screen
S32_ALTHEXOUT 		equ	 39d	; Write number to alternate screen
S32_TEXTIN 		equ	 40d	; Read text
S32_SETCURSOR 		equ	 41d	; Cursor Flashing On/Off
S32_GETINTVECT 		equ	 42d	; Get Interrupt Vector
S32_GETSELECTORBASE 	equ	 43d	; Get Selector's base address
S32_SETSELECTORLIMIT 	equ	 44d	; Set Selector limit

;----------------------------------------------------------------------------
; screen constants
;----------------------------------------------------------------------------
SCR_TERM	equ	0	; Terminal Screen
SCR_DEBUG	equ	1	; Debug Screen
SCR_BOOTUP	equ	2	; Bootup Screen
SCR_MODLOAD	equ	3	; Modload Screen
SCR_CRASH	equ	4	; Crash Screen

;----------------------------------------------------------------------------
; System16 Info Block
; ~~~~~~~~~~~~~~~~~~~
; This is the System16 Info Block. It's content is mostly filled in by
; System16 at bootup. The values here are for system use, but can be
; usefull for some appliciations aswell... This table is located IN the
; code-segment of System16.
; System16 will always be loaded at 500h. It starts with 100h zeros. So
; to get for example the physical address of the KeyboardArea, this
; assemblercode could retrieve it: mov eax, gs:[600h+74]
; Provided that GS is loaded with the descriptor that covers memory
; 00000000 to 4Gig (As it always should under V2_OS).
;----------------------------------------------------------------------------
S16InfoBlock    STRUC
        db 4 dup(?) ; nothing here
        S16_S16Size     db ?    ; This value is used by Boot.com to find out
                                ; how many sectors to load into memory before
                                ; passing control to System16. Multiply this
                                ; value by 512 to get the size of System16 in
                                ; bytes.
        S16_S32Size     db ?    ; This value is used by System16 to find out
                                ; how many sectors to load into memory before
                                ; passing control to System32. Multiply this
                                ; value by 512 to get the size of System32 in
                                ; bytes.
        S16_S16Addr     dd ?    ; This value is the physical Base address of
                                ; System16's code Descriptor. This value is
                                ; currenly fixed to 500h
                                ; Note that System16's code starts with 100h
                                ; bytes of zero's.
        S16_S32Addr     dd ?    ; This value is the physical Base address of
                                ; System32's code Descriptor. It starts right
                                ; after System16's code. Note that System32
                                ; also starts with 100h bytes of zero's.
        S16_RMPortalSeg dw ?    ; The RealMode Portal Segment
        S16_RMPortalOffs        dw ?    ; The RealMode Portal Offset
        S16_IDTLen      dw ?    ; The IDT Length (in Bytes)
        S16_IDTBase             dd ?    ; The IDT physical base address
        S16_GDTLen      dw ?    ; The GDT Length (in Bytes)
        S16_GDTBase             dd ?    ; The GDT physical base address
        S16_StackBase   dd ?    ; The physical base address of the 'Shared
                                ; Stack-Space'
        S16_StackSize   dd ?    ; Size in bytes of the 'Shared Stack-Space'
        S16_S16FOffs    dw ?    ; System16Function Offset
        S16_S16FRetAddr dd ?    ; System16Function's Return Address
        S16_RealEAX     dd ?    ; Real_EAX
        S16_RealEBX     dd ?    ; Real_EBX
        S16_RealECX     dd ?    ; Real_ECX
        S16_RealEDX     dd ?    ; Real_EDX
        S16_RealESI     dd ?    ; Real_ESI
        S16_RealEDI     dd ?    ; Real_EDI
        S16_RealDS              dw ?    ; Real_DS
        S16_RealES              dw ?    ; Real_ES
        S16_RealFS              dw ?    ; Real_FS
        S16_KbdAreaBase dd ?    ; The physical base address of the 'Keyboard
                                ; Area'
        S16_BootupGDT   dw ?    ; Bootup GDT
        S16_GDTEntries  dw ?    ; GDT_Entries
        S16_DiskBuffSeg dw ?    ; Diskbuffer Segment
        S16_DiskBuffSize        dd ?    ; Diskbuffer Size in bytes
        S16_FixedDisksCnt       dw ?    ; Number of Fixed Disks
        S16_DIBOffs     dw ?    ; Offset to the DiskInfoBlock (Relative to
                                ; System16's Base Address)
                        dw ?    ; Reserved
        S16_BootDisk    db ?    ; (BIOS)Drive number of the disk that V2_OS
                                ; Booted from
        S16_CMOSImageAddr       dd ?    ; Address of the CMOS
                                ; data-image(Copied from NVRam by
                                ; System16 at bootup)
                        db ?    ; Reserved
        S16_FoundCyl    dd ?    ; Found_Cylinder
        S16_FoundHead   dd ?    ; Found_Head
        S16_FoundSec    dd ?    ; Found_Sector
        S16_PartListOffs        dw ?    ; Offset to the
                                ; PartitionList(Relative to
                                ; System16's Base Address)
        S16_FlopCnt     dw ?    ; Number of floppydrives present in the
                                ; system
        S16_ModListAddr dd ?    ; ModList Address
        S16_ModListSize dd ?    ; ModList Size in bytes
        S16_ServListAddr        dd ?    ; ServerList Address
        S16_ServListSize        dd ?    ; ServerList Size in Bytes
                        db 128 dup(?)   ; ... Reserved
S16InfoBlock    ENDS

;----------------------------------------------------------------------------
; Disk Info Block
; ~~~~~~~~~~~~~~~
; In the System16 Info Block, there is the entry
; 'Offset to the DiskInfoBlock (Relative to System16's Base Address)
; ' at offset 90, that points to this block. It's a chained structure
; with information on all diskdrives accessable to the V2_OS. Each entry
; is 32bytes long and starts with one byte that holds the 'DiskNumber'.
; This is 0 for the first floppy, 80h for the first fixed disk, 81h for
; the second fixed disk etc. If this value is 0ffh, this indicates the
; end of the chain.
;----------------------------------------------------------------------------
DiskInfoBlock   STRUC
        DIB_DiskNum     db ?    ; DiskNumber
                db ?    ; Reserved (for DiskType)
        DIB_MaxCyl      dw ?    ; Maximum Cylinders
        DIB_MaxHead     db ?    ; Maximum Heads
        DIB_MaxSec      db ?    ; Maximum Sectors
        DIB_DiskIdx     db ?    ; DiskIndex in the 'Type Chain', like 'IDE3',
                        ; or 'FLOPPY2' etc.
                db ?    ; Reserved
        DIB_Type        db 8 dup(?)     ; Type BIOS, IDE, ATAPI etc. Padded
                        ; with spaces !!!, left-alligned
        DIB_ServMod     db 8 dup(?)     ; Serving Module
        DIB_DiskName    dd ?    ; DiskName like 'fd0',0 or 'hdb',0 etc.
                        ; 0-Terminated
                dd ?    ; Reserved
DiskInfoBlock   ENDS

;----------------------------------------------------------------------------
; The PartList
; ~~~~~~~~~~~~
; In the System16 Info Block, there is the entry
; 'Offset to the PartitionList (Relative to System16's Base Address)
; ' at offset 112, that points to this block. It's a chained structure
; with information on all partitions accessable to the V2_OS. Each entry
; is 32bytes long.
;----------------------------------------------------------------------------
PartListEntry   STRUC
        PL_PartName     db 6 dup(?)     ; Partition Name, lowercase,
                                ; terminated with a zero. Example'hda1',0 or
                                ; 'fd0',0 or 'hdb',0 etc.
                                ; reserved
        PL_PartDiskNum  db ?    ; Physical Disknumber (from the
                                ; DiskList)
        PL_PartStartSec dd ?    ; First sector of this partition (in LBA)
        PL_PartSize     dd ?    ; Amount of sectors in this partition
                db ?    ; Reserved
        PL_PartDiskName db 4 dup(?)     ; DiskName like 'fd0',0 or 'hdb'
                                ; etc. (0 terminated)
                db 12 dup(?)    ; ... Reserved
PartListEntry   ENDS

;----------------------------------------------------------------------------
; The KeyboardArea and Buffer
; ~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Everytime a user presses a key, an IRQ1 is fired. The default handler
; for IRQ1 asks the keyboardcontroller what scancode was pressed or
; released. This event is recorded in the Key ScanCodeMap. Every key
; that is pressed has a value of 1, and a zero if released. The handler
; then translates this scancode to an ascii character where possible,
; and checks the status of the shift-keys to see if it needs to be upper
; cased. Then the ascii character is added to the keyboardbuffer.
;----------------------------------------------------------------------------

KbdArea STRUC
        KBD_ScanCodeMap db 255 dup(?)   ; pressed keys have 1, 0 otherwise
        KBD_BufferSize db ?
        KBD_CharsInBuffer db ?
        KBD_Buffer db 255 dup (?)       ; actual size is determined by
                                        ; KBD_BufferSize field
KbdArea ENDS

;----------------------------------------------------------------------------
; The ModuleList
;
; In the System16 Info Block, there is the entry 'ModList Address' at
; offset 116, that points to this block. It's a chained structure with
; information on all modules that are currently loaded into memory and
; have been successfully initialized. Each entry is 128bytes long.
;----------------------------------------------------------------------------

ModListEntry    STRUC
        ML_ModID        db 8 dup (?)    ; ModID
        ML_CodeDesc     dw ?    ; Primary CodeDescriptor
        ML_DataDesc     dw ?    ; Primary DataDescriptor
        ML_ModAddr      dd ?    ; Absolute address in memory
        ML_ModSize      dd ?    ; Size of this module in bytes
        ML_ModLoad      dd ?    ; Offset to the 'Mod_Load' Code
        ML_ModUNLoad    dd ?    ; Offset to the 'Mod_UNLoad' Code
        ML_ServiceTable dd ?    ; Offset to the 'Service Table'
        ML_ModuleName db 32 dup (?)     ; Module Name
                db 64 dup (?)   ; ... Reserved
ModListEntry    ENDS

;----------------------------------------------------------------------------
; The ServerList
;
; In the System16 Info Block, there is the entry 'ServerList Address' at
; offset 124, that points to this block. It's a chained structure with
; information on all Servers that are currently loaded into memory and
; have been successfully initialized. Each entry is 64bytes long.
;----------------------------------------------------------------------------

ServListEntry   STRUC
        SL_ServModID    db 8 dup (?)    ; ModID that handles this server's
                                        ; requests
        SL_ServType     db 16 dup (?)   ; example 'VideoCard',0 or 'Nic',0
                                        ; etc.
                db 40 dup (?)   ;  ... Reserved
ServListEntry   ENDS

