;=============================================================
; SBISR
; -----
; SoundBlaster Interrupt Service Routine
; this interrupt is called after each completed dma transfer
; 
;=============================================================

ISRMsg	  db 'I.',0
StopMsg	  db 'S',0

V2S_ISR:
	cli
	pusha
	push ds

	mov	ax,[cs:DataSel]			; get our dataselector so we can update vars
	mov	ds,ax

	mov	dx,[SBPort]			; ACk interrupt to SB by reading port 2xEh
	add	dx,0eh
	in	al,dx
	inc	dx
	in	al,dx

	mov	al,[SBIRQ]			; Send End-Of-Interrupt to PIC's
	and	al,0F8h
	jz	V2S_ISR_AckPIC1			; IRQ > 7 ?
	mov	dx,PIC2				; Yes, also ACK interrupt to PIC2
	mov	al,EOI
	out	dx,al
V2S_ISR_AckPIC1:	
	mov	dx,PIC1				; ACK interrupt to PIC1
	mov	al,EOI
	out	dx,al


	cmp	byte [DoIRQDetection],0		; Are we doing IRQ detection (only used by sbdetect.asm)
	je	V2S_ISR_NoDetect
	
	mov	byte [IRQDetected],1
	jmp	V2S_ISR_END
	
V2S_ISR_NoDetect:

	cmp	byte [V2S_StopDMA],0		; Stop playing ?
	jne	V2S_ISR_CONTINUE		; no, continue

	mov	byte [V2SPS_Streaming],0	; stop streaming
	call	DSPDMAExit
	jmp	V2S_ISR_END			; exit

V2S_ISR_CONTINUE:

	call	DSPDMARepeat

	cmp	byte [V2S_CurBuffer],0		; which buffer just finished?
	je	V2S_ISR_Buffer1			; 0 = buffer 1
	mov	edi,[DMABufferOf2]		; refill buffer 2
	mov	byte [V2S_CurBuffer],0		; current buffer playing is 0 (buffer 1)
	jmp	V2S_ISR_CB
V2S_ISR_Buffer1:
	mov	edi,[DMABufferOf1]		; refill buffer 1
	mov	byte [V2S_CurBuffer],1		; current buffer playing is 1 (buffer 2)
V2S_ISR_CB:
	mov	ecx,[DMABufferSiz]		; ecx number of bytes we want
	call	far [V2SPS_Callback]		; call callback function
	or	eax, eax			; 0 bytes copied?
	setnz	byte [V2S_StopDMA]		; if eax=0, then V2S_StopDMA is set to 0, else V2S_StopDMA = 1

V2S_ISR_END:
	pop	ds
	popa
	sti
	iret
	

;=============================================================
; Setup and enable SB interrupt handler
;=============================================================

HookSBint:
	push eax
	push ebx
	push ecx
	push edx
	push edi

        xor  ax,ax
	mov  al,[SBIRQ]
	mov  di,ax        
	mov  ah,[IRQtoInt + di]		; map IRQ to an interrupt
        mov  al,2ah			; save old int
	int  20h
	mov  word [saveIntSel],bx
	mov  dword [saveIntOff],edi
	

        xor  ax,ax
	mov  al,[SBIRQ]
	mov  di,ax        
	mov  ah,[IRQtoInt + di]		; map IRQ to an interrupt
	mov  al,1			; set new int
	mov  bx,cs
	mov  edi,V2S_ISR
        int  20h

	mov  al,[SBIRQ]			; IRQ > 7?
	and  al,0F8h
	jnz  HookSBInt_OnPIC2		; yes, select PIC2
	mov  dx,PIC1+1			; else select PIC1
	mov  cl,[SBIRQ]
	jmp  HookSBInt_EnableInt
HookSBInt_OnPIC2:
	mov  dx,PIC2+1
	mov  cl,[SBIRQ]
	sub  cl,8
HookSBInt_EnableInt:
	mov  bl,1			; enable interrupt
	shl  bl,cl
	not  bl
	in   al,dx
	and  al,bl
	out  dx,al

	pop  edi
	pop  edx
	pop  ecx
	pop  ebx
	pop  eax
	ret

;=============================================================
; Restore old interrupt handler
;=============================================================
RestoreSBint:
	push eax
	push ebx
	push ecx
	push edx
	push edi

	xor  ax,ax
	mov  al,[SBIRQ]
	mov  di,ax
	mov  ah,[IRQtoInt + di]		; map IRQ to an interrupt
	mov  al,1
	mov  bx,saveIntSel		; restore saved int
	mov  edi,saveIntOff
	int  20h

	mov  al,[SBIRQ]			; IRQ > 7?
	and  al,0F8h
	jnz  RestoreSBInt_OnPIC2	; yes, select PIC2
	mov  dx,PIC1+1			; else select PIC1
	mov  cl,[SBIRQ]	
	jmp  RestoreSBInt_DisableInt
	
RestoreSBInt_OnPIC2:
	mov  dx,PIC2+1
	mov  cl,[SBIRQ]
	sub  cl,8
	
RestoreSBInt_DisableInt:
	mov  bl,1			; disable interrupt
	shl  bl,cl
	in   al,dx
	or   al,bl
	out  dx,al

	pop  edi
	pop  edx
	pop  ecx
	pop  ebx
	pop  eax
	ret