; TODO:  error checking on input params (?)
;
; 

;==================================================================
; Service 15h - PlayStream
; ------------------------
; IN :	es:edx	- pointer to callback function
;     	ebx     - samplerate
;       cl	- bits (8/16)
;       ch      - mode (0=mono, 1=stereo)
; OUT:  ax	- 0 on success, otherwise error
;
; The callback function is called when a buffer needs refilling.
; Callback function:
; IN : 	edi     - Abs. address of buffer
;	ecx	- Number of bytes needed
; OUT:  eax	- Number bytes put in buffer (eax=0 stops streaming)
;===================================================================

V2S_PlayStream:
	push	ebx
	push	ecx
	push	edx
	push	edi
	push 	esi
	push	ds
	
	cmp	byte [V2SPS_Streaming],0		; are we already playing ?
	je	V2S_PlayStream_New			; no start new stream
	
	mov	ax,0ffffh				; ax = errorcode
	jmp	V2S_PlayStream_End			; exit
	
V2S_PlayStream_New:
	mov	al,1
	call	DSPSetSpeaker

	mov	byte [V2S_StopDMA],1			; 
	mov	word [V2SPS_CallbackSel],es
	mov	dword[V2SPS_CallbackOff],edx
	mov	[DSPBits],cl	
	mov	[DSPChannels],ch

        mov	dx,[SBPort]		 		; 
	call	DSPSetSampleRate	 		; set sample rate to ebx
	mov	byte [V2S_CurBuffer],0	 		; start with buffer 1
	
	mov	edi,[DMABufferOf1]			; Abs. address of buffer 1
	mov	ecx,[DMABufferSiz]			; Size of one buffer
	shl	ecx,1					; *2 (fill both buffers)
	call	far [V2SPS_Callback]			; fill both buffers

; program dma for entire buffer
	cmp	cl,8
	jne	V2SPS_DMA16
V2SPS_DMA8:
	mov	edi,[DMABufferOf1]			; Abs. address of first buffer
        mov	al,dmaModeSINGLE			; setup DMA transfer of sound data
        or	al,dmaRdTransfer			; SB reads from memory
        or   	al,dmaEnableAutoInit			; Autoinit -> no need to program dma again and again
	mov	ah,[SBLoDMA]				; Use this DMA channel
	mov	ecx,[DMABufferSiz]			; size of block-1
	dec	ecx
	shl	ecx,1					; *2 (DMA always plays entire buffer)
	call	DMAInitTransfer	 			; 

; program sb for half buffer -> double buffering
; when the first half is played the sb triggers an interrupt, allowing us to refill the first half just played
; while restarting playing with second half of buffer

	mov	ecx,[DMABufferSiz]			; Buffer size-1
	dec	ecx
	call	DSPDMADAC8
	jmp	V2S_PlayStream_End

V2SPS_DMA16:
	mov	edi,[DMABufferOf1]			; Abs. address of first buffer
        mov	al,dmaModeSINGLE			; setup DMA transfer of sound data
        or	al,dmaRdTransfer			; SB reads from memory
        or   	al,dmaEnableAutoInit			; Autoinit -> no need to program dma again and again
	mov	ah,[SBHiDMA]				; Use this DMA channel
	mov	ecx,[DMABufferSiz]			; size of block-1 (in words)
	dec	ecx
	call	DMAInitTransfer	 			; 

; program sb for half buffer -> double buffering
; when the first half is played the sb triggers an interrupt, allowing us to refill the first half just played
; while restarting playing with second half of buffer

	mov	ecx,[DMABufferSiz]			; Buffer size-1
	dec	ecx
	call	DSPDMADAC16
V2S_PlayStream_End:

	pop	ds
	pop	esi
	pop	edi
	pop	edx
	pop	ecx
	pop	ebx
	retf



;==================================================================
; Service 16h - StopStream
; ------------------------
; IN :
;	Nothing
; OUT:
;	Nothing
;
;===================================================================


V2S_StopStream:
	call	DSPDMAExit
	mov	byte [V2SPS_Playing],0
	mov	byte [V2SPS_Streaming],0
	retf


; vars
	V2SPS_Playing		db	0
	V2SPS_Streaming		db	0
V2SPS_Callback	:
	V2SPS_CallbackOff	dd	0	
	V2SPS_CallbackSel	dw	0	
	
	PSMsg	DB 'PlayStream Called',13,10,0
