;============================================================
; SoundBlaster Autodetect Routine
;
; Author   : Bjorge Dijkstra (bjorge@gmx.net)
; Date     : 03-03-2000
; version  : 1.0
;============================================================
	
	
;=============================================================
; TestDMAIRQCombo
;=============================================================
	Sound:
	times 100 DB 80h	 	; test buffer 
DET_TestDMAIRQCombo:
 	push	eax
 	push	ebx
 	push	ecx
 	push	edi
 	push	esi
 	
 	stc				 ; assume failure	
 	mov	byte [IRQDetected],0
 	mov	byte [DoIRQDetection],1
	call	HookSBint		 ; Hook SB interrupt

	mov	ebx,8000		 ; Set sample rate to 8KHz
	call	DSPSetSampleRate
	
	mov	bx,cs		 	 ; Get base address of selector cs
	mov	al,2bh
	int	20h
	add	edi,Sound	 ; Add offset to test data
	

        mov	al,dmaModeSINGLE	 ; setup DMA transfer of sound data
        or	al,dmaRdTransfer 	 ; SB reads from memory
	mov	ah,[SBLoDMA]		 ; Use this DMA channel
	mov	cx,100	 	 	 ; size of block
	call	DMAInitTransfer	 	 ; 

	mov	cx,100
	mov	al,014h			; setup SB for 8-bit Mono DMA transfer	
	call	DSPWrite		; write command
	mov	al,cl			; block size lower byte
	call	DSPWrite		; 
	mov	al,ch			; block size higher byte
	call	DSPWrite		; playing starts immediately after writing this byte

	mov	al,1ah			; delay
	mov	edi,2			; wait 50 msec
	int	20h			; transfer should take about 10-15 msec

	call	RestoreSBint
 	mov	byte [DoIRQDetection],0
	pop	esi		
	pop	edi
	pop	ecx
	pop	ebx
	pop	eax
	ret 
 

;=============================================================
; FindIRQandLoDMA (used if it's not a SB16+)
;============================================================= 
DET_FindIRQandLoDMA:
	push	eax
	push	ebx
	push	ecx
	push	edx
	push	edi
	push	esi
	
	mov	di,0			; DMA index
	mov	si,0			; IRQ index
	
FILD_Testloop:

	mov	al,[LoDMAs+di]		; select a DMA channel
	mov	byte [SBLoDMA],al
	
	
	mov	al,[IRQs+si]		; select an IRQ
	mov	byte [SBIRQ],al
	
	call 	DET_TestDMAIRQCombo		; test 'em
	
	cmp	byte [IRQDetected],1		; found ?
	je	FILD_Testsuccess		; yes, we're done!
	
	inc	si			; no, next irq
	cmp	si,6			; tried all irq's?
	jl	FILD_Testloop
	
	mov	si,0			; then next dma
	inc	di
	cmp	di,3			; tried all dma's?
	jge	FILD_Testfailed		; then we didn't find anything
	jmp	FILD_Testloop
	
FILD_Testfailed:
	stc
        jmp	FILD_End

FILD_Testsuccess:
	clc
	
FILD_End:
	pop	esi
	pop	edi
	pop	edx
	pop	ecx
	pop	ebx
	pop	eax	
 	ret
 	
 
;=============================================================
; SB16FindIRQandDMA
;=============================================================  
DET_SB16FindIRQandDMA:
	push	eax			; save used registers
	push	ebx
	push	ecx
	push	edx
	push	edi
	
	mov	dx,[SBPort]		
	add	dx,04h			; select mixer index port
	mov	al,80h			; irq select register
	out	dx,al			; select

	xor	ax,ax
	inc	dx			; select mixer data port
	in	al,dx			; read irq select register
	
	mov	di,0		
	mov	cx,4			; get irq setting from al
IRQloop:
	test	al,1			; al : 01h = irq 2
	jnz	IRQOk			;      02h = irq 5
	shr	al,1			;      04h = irq 7
	inc	di			;      08h = irq 10
	loop	IRQloop
IRQOk:	
	mov	al,[SB16IRQs + di]
	mov	byte [SBIRQ],al
	
	
	mov	dx,[SBPort]
	add	dx,04h			; select mixer index port
	mov	al,81h			; dma select register
	out	dx,al			; select
	
	xor	ax,ax
	inc	dx			; select mixer data port
	in	al,dx			; read dma select register
	out     dx,al
	and	al,11101011b		; mask out reserved dma's 2,4
	
	mov	bl,al			; al = LoDMA setting, bl = HiDMA setting
	shr	bl,4
	mov	cx,4
	mov	di,0			; get LoDMA setting from al
LoDMAloop:
	test	al,1			; al : 01h = dma 0
	jnz	LoDMAOk			;      02h = dma 1
	shr	al,1			;      04h = dma 2 (reserved)
	inc	di			;      08h = dma 3
	loop	LoDMAloop
LoDMAOk:		
	mov	al,[SB16LoDMAs + di]
	mov	byte [SBLoDMA],al

	mov	cx,4
	mov	di,0			; get HiDMA from bl
HiDMAloop:
	test	bl,1			; bl : 01h = dma 4 (reserved)
	jnz	HiDMAOk			;      02h = dma 5
	shr	bl,1			;      04h = dma 6
	inc	di			;      05h = dma 7
	loop	HiDMAloop
HiDMAOk:	
	mov	al,[SB16HiDMAs + di]
	mov	byte [SBHiDMA],al	
	
	pop	edi
	pop	edx
	pop	ecx
	pop	ebx
	pop	eax
	ret
	
;=============================================================
; GetSBModel
;=============================================================  	
GetSBModel:
	push	eax
	
	mov	ah,[DSPVMajor]
	mov	al,[DSPVMinor]
	
	cmp	ah,1		; 1.x ?
	jne	GSM_M2
	mov	dword [SBModel], SBModel1x
	jmp	GSM_End

GSM_M2:
	cmp	ah,2
	jne	GSM_M3
	mov	dword [SBModel], SBModel20
	jmp	GSM_End
	
GSM_M3:
	cmp	ah,3
	jne	GSM_M4
	cmp	al,0
	jne	GSM_M3b
	mov	dword [SBModel], SBModelPro	
	jmp	GSM_End
GSM_M3b:
	mov	dword [SBModel], SBModelPro2
	jmp	GSM_End
	
GSM_M4:
	cmp	ah,4
	jne	GSM_UnkModel
	cmp	al,11
	jge	GSM_M4b
	mov	dword [SBModel], SBModelSB16
	jmp	GSM_End
GSM_M4b:
	cmp	al,12
	jge	GSM_M4c
	mov	dword [SBModel], SBModelSB162
	jmp	GSM_End
GSM_M4c:
	cmp	al,12
	jge	GSM_M4d
	mov	dword [SBModel], SBModelAWE32
	jmp	GSM_End
GSM_M4d:
	cmp	al,10h
	jne	GSM_UnkModel
	mov	dword [SBModel], SBModelAWE64
	jmp	GSM_End
GSM_UnkModel:
	mov	dword [SBModel], SBUnkModel
	
GSM_End:
	pop	eax
	ret	
	

;=============================================================
;=============================================================
;
;  MAIN
;
;=============================================================
;=============================================================

V2S_DetectSB:
	mov	ah,81h
	call	DSPDetectBase		; find base port
	jc	SBDetect_Failed		; can't find base port..

	mov	[SBPort],dx
	call	DSPGetVersion
	
	call	GetSBModel
	
	cmp	ah,4			; Is it a SB16+ ?
	jl	V2SD_NOSB16		; no then try alternative
	
	call	DET_SB16FindIRQandDMA	; read SB16 settings from mixer chip
	
	jmp	V2SD_Exit
	
V2SD_NOSB16:	
	mov	al,0	
	call	DSPSetSpeaker		; Turn speakers off, because we're going to output
					; some test data, which can be noisy
	mov	ah,82h	
        call    DET_FindIRQandLoDMA		; Find IRQ and LoDMA
        jc	SBDetect_Failed
        
V2SD_Exit:
		
	mov	ah,0h
	ret
	        
SBDetect_Failed:
	cmp	al,81h
	jne	V2SD_Err82
	push	eax
	push	edi
	mov	al,04h
	mov	edi,Err81Msg
	int	20h
	pop	edi
	pop	eax
	ret
V2SD_Err82:
	push	eax
	push	edi
	mov	al,04h
	mov	edi,Err82Msg
	int	20h
	pop	edi
	pop	eax
	ret	

Err81Msg	db	'Failed detecting SB Base port',13,10,0
Err82Msg	db	'Found SB Base port, but failed to detect the IRQ and DMA settings',13,10,0
