[bits 32]
org 100h

; First, the header.
	jmp	run			; Catch people 'running' the module so we just tell them to load it...
	times 3	db	0		; Fill up to 08h, but not 08h

	db	'V2MD'			; Abbracadabra, a magic word
	db	'V2_SOUND'		; Module ID (8 characters)
	db	0,03			; Version number (0.0.3)
	db	0,1			; ????
	
	dd	MOD_LOAD		; pointer to MOD_LOAD
	dd	MOD_UNLOAD		; pointer to MOD_UNLOAD
	dd	MOD_TABLE		; pointer to MOD_TABLE

	dd	MOD_NAME
	dd	MOD_DESCRIPTION
	dd	MOD_COPYRIGHT

; Then some strings
	MOD_NAME	db	'V2OS Sound Module 0.0.3', 0
	MOD_DESCRIPTION	db	'Sound Module for SB and compatibles', 0
	MOD_COPYRIGHT	db	'30-04-2000', 0
	runMsg		db	'This is a "module" and cannot be run', 0Dh, 0Ah
			db	"Please, use the 'loadmod' command instead!", 0Dh, 0Ah, 0
	loadMsg		db	'V2SOUND 0.0.3 Loading...', 0Dh, 0Ah, 0
	unloadMsg	db	'V2SOUND Unloading...', 0Dh, 0Ah, 0
	errorMsg	db	'An error occurred : ',0
	CRLF		db	13,10,0
	
; some constants
	PIC1		equ  020h	; First  PIC (handles IRQ 0-7)
	PIC2		equ  0A0h	; Second PIC (handles IRQ 8-15)
	EOI		equ  020h	; End-Of-Interrupt cmd

; data
	
	saveIntSel	DW 0
        saveIntOff	DD 0
	IRQDetected	DB 0
	DoIRQDetection	DB 0
		
        IRQs		DB 07h,05h,0Ah,02h		; irq's to check in preferred order
        LoDMAs		DB 01h,03h,00h			; dma's to check in preferred order        

        SB16IRQs	DB 02h,05h,07h,0Ah		; possible irq's on sb
        SB16LoDMAs	DB 00h,01h,02h,03h		; possible dma's
        SB16HiDMAs	DB 04h,05h,06h,07h		;
        
	IRQtoInt	DB 08h,09h,0Ah,0Bh,0Ch,0Dh,0Eh,0Fh,70h,71h,72h,73h,74h,75h,76h,77h
	
	SBModel		DD 0				; offset to modelstring
	SBPort		DW 240h
	SBIRQ		DB 5
	SBLoDMA		DB 3
	SBHiDMA		DB 5
	
	CapBitSize	DB 16				; max. sample bitsize (8 or 16)
	CapChannels	DB 0				; max. channels (1 = mono, 2=stereo)
	CapSamplerate	DW 44100			; max. samplerate
	CapMixer	DB 0				; mixer available (0=no, 1=yes) SBPro and up
	
	DMABufferLin	DD 0
	DMABufferSel	DW 0
	DMABufferOf1	DD 0
	DMABufferOf2	DD 0
	DMABufferSiz	DD 16000
	V2S_CurBuffer	DB 0
	V2S_StopDMA	DB 0
	V2S_InitOK	DB 0FFh
	DataSel		DW 0



; And then the routines

%include "sbdsp.asm"			; Various SB DSP routines
%include "dma.asm"			; DMA transfer routines

%include "v2s_isr.asm"			; Interrupt Service Routine
%include "v2s_showsettings.asm"		; Display settings at startup
%include "v2s_sbdetect.asm"		; SB Detect routine (used in MOD_LOAD)
%include "v2s_speakers.asm"		; Service 13h - SetSpeakers
%include "v2s_playstream.asm"		; Service 15h - PlayStream and 16h - StopStream


run:					; This sould just put the normal error message.
	mov	al, 04h
	mov	edi, runMsg
	int	20h			; Print String
	retf

reserved:				; When a non-existing service is called !
	retf

	
V2S_AllocateDMABuffer:
	mov	al,0Eh			; Allocate Mem
	mov	ebx, [DMABufferSiz]
	shl	ebx,1			; allocate 2*size
	mov	dx,0100h
	int	20h
	mov	dword [DMABufferLin], edi
	mov	word  [DMABufferSel], cx
	mov	dword [DMABufferOf1], edi	; linear address buffer part 1
	add	edi,  [DMABufferSiz]
	mov	dword [DMABufferOf2], edi	; linear address buffer part 2
	ret

	
V2S_DeAllocateDMABuffer:
	mov	al,0fh
	mov	bx,[DMABufferSel]
	int	20h
	ret

MOD_LOAD:				
	mov	word [DataSel], ds

	mov	al, 04h		
	mov	edi, loadMsg
	int	20h			; Print string
	
	call	V2S_DetectSB
	cmp	ah,0
	jne	MOD_LOAD_ERROR
	
	mov	dx,[SBPort]
	mov	al,[SBLoDMA]
	mov	ah,[SBHiDMA]
	mov	bl,[SBIRQ]
	call	DSPInit
	call	ShowSettings		; print sb settings
	call	V2S_AllocateDMABuffer	; allocate mem buffer for DMA
	call	HookSBint
	
	mov	byte [V2S_InitOK],0
	retf
	
MOD_LOAD_ERROR:
	push	ax
	mov	al,04h
	mov	edi, errorMsg
	int	20h
	pop	ax
	mov	bl,ah
	mov	ax,0105h
	int	20h
	mov	al,04h
	mov	edi, CRLF
	int	20h
	retf
	

MOD_UNLOAD:
	call	V2S_DeAllocateDMABuffer	; 
	mov	al, 04h			; 
	mov	edi, unloadMsg		; 
	int	20h			; Print string
	retf

; This is where we insert our Module's services


V2S_CheckV2S:
	mov	al,[V2S_InitOK]
	retf

; And finally the service table (don't forget to fill IT!!!!)

MOD_TABLE:
	dd	V2S_CheckV2S		; Service 00h	-	( moved, please use service 10h )
	dd	reserved		; Service 01h
	dd	reserved
	dd	reserved
	dd	reserved
	dd	V2S_PlayStream		; service 05h	-	( moved, please use service 15h )
	dd	reserved
	dd	reserved
	dd	reserved
	dd	reserved
	dd	reserved
	dd	reserved
	dd	reserved
	dd	reserved
	dd	reserved
	dd	reserved		; Service 0Fh
	dd	V2S_CheckV2S		; Service 10h	-	Checks if v2sound was init'ed properly (al=0=OK)
	dd	reserved
	dd	reserved
	dd	V2S_SetSpeakers		; Service 13h	-	Turn speakers on/off (al=0 on, else off)
	dd	reserved
	dd	V2S_PlayStream		; service 15h	-	Start playback of a stream, es:edx address of callback
	dd	V2S_StopStream		; service 16h	-	Stop playback of a stream
	dd	reserved
	dd	0			; Just to make sure we don't get
	dd	0			; an unintended service in here 
	dd	0			; somewhere.







